@@ -52,7 +52,7 @@
 #include "ns3/network-module.h"
 #include "ns3/applications-module.h"
 #include "ns3/mobility-module.h"
-#include "ns3/tools-module.h"
+#include "ns3/stats-module.h"
 #include "ns3/random-variable-stream.h"
 #include "ns3/wifi-module.h"
 #include "ns3/internet-module.h"
@@ -64,10 +64,10 @@
 #include <iostream>
 #include <fstream>
 
-NS_LOG_COMPONENT_DEFINE ("multirate");
-
 using namespace ns3;
 
+NS_LOG_COMPONENT_DEFINE ("multirate");
+
 class Experiment
 {
 public:
@@ -103,6 +103,7 @@ private:
 
   double totalTime; 
   double expMean;
+  double samplingPeriod;
 
   uint32_t bytesTotal;
   uint32_t packetSize;
@@ -129,6 +130,7 @@ Experiment::Experiment (std::string name) :
   m_output (name),
   totalTime (0.3),
   expMean (0.1), //flows being exponentially distributed
+  samplingPeriod(0.1),
   bytesTotal (0),
   packetSize (2000),
   gridSize (10), //10x10 grid  for a total of 100 nodes
@@ -172,12 +174,12 @@ Experiment::ReceivePacket (Ptr<Socket> socket)
 void
 Experiment::CheckThroughput ()
 {
-  double mbs = ((bytesTotal * 8.0) /1000000);
+  double mbs = ((bytesTotal * 8.0) /1000000 /samplingPeriod);
   bytesTotal = 0;
   m_output.Add ((Simulator::Now ()).GetSeconds (), mbs);
 
-  //check throughput every 1/10 of a second 
-  Simulator::Schedule (Seconds (0.1), &Experiment::CheckThroughput, this);
+  //check throughput every samplingPeriod second
+  Simulator::Schedule (Seconds (samplingPeriod), &Experiment::CheckThroughput, this);
 }
 
 /**
@@ -309,14 +311,14 @@ Experiment::SendMultiDestinations (Ptr<Node> sender, NodeContainer c)
     }
 }
 
-Vector
+static inline Vector
 GetPosition (Ptr<Node> node)
 {
   Ptr<MobilityModel> mobility = node->GetObject<MobilityModel> ();
   return mobility->GetPosition ();
 }
 
-std::string
+static inline std::string
 PrintPosition (Ptr<Node> client, Ptr<Node> server)
 {
   Vector serverPos = GetPosition (server);
@@ -506,12 +508,11 @@ Experiment::Run (const WifiHelper &wifi, const YansWifiPhyHelper &wifiPhy,
       phy.EnableAsciiAll (ascii.CreateFileStream (GetOutputFileName () + ".tr"));
     }
 
-  Ptr<FlowMonitor> flowmon;
+  FlowMonitorHelper flowmonHelper;
 
   if (enableFlowMon)
     {
-      FlowMonitorHelper flowmonHelper;
-      flowmon = flowmonHelper.InstallAll ();
+      flowmonHelper.InstallAll ();
     }
 
   Simulator::Stop (Seconds (totalTime));
@@ -519,7 +520,7 @@ Experiment::Run (const WifiHelper &wifi, const YansWifiPhyHelper &wifiPhy,
 
   if (enableFlowMon)
     {
-      flowmon->SerializeToXmlFile ((GetOutputFileName () + ".flomon"), false, false);
+      flowmonHelper.SerializeToXmlFile ((GetOutputFileName () + ".flomon"), false, false);
     }
 
   Simulator::Destroy ();
@@ -534,6 +535,17 @@ Experiment::CommandSetup (int argc, char **argv)
   CommandLine cmd;
   cmd.AddValue ("packetSize", "packet size", packetSize);
   cmd.AddValue ("totalTime", "simulation time", totalTime);
+  // according to totalTime, select an appropriate samplingPeriod automatically.
+  if (totalTime < 1.0)
+    {
+	  samplingPeriod = 0.1;
+    }
+  else
+    {
+	  samplingPeriod = 1.0;
+    }
+  // or user selects a samplingPeriod.
+  cmd.AddValue ("samplingPeriod", "sampling period", samplingPeriod);
   cmd.AddValue ("rtsThreshold", "rts threshold", rtsThreshold);
   cmd.AddValue ("rateManager", "type of rate", rateManager);
   cmd.AddValue ("outputFileName", "output filename", outputFileName);
