@@ -26,6 +26,7 @@
 #include "ns3/address.h"
 #include "ns3/inet-socket-address.h"
 #include "ns3/inet6-socket-address.h"
+#include "ns3/packet-socket-address.h"
 #include "ns3/node.h"
 #include "ns3/nstime.h"
 #include "ns3/data-rate.h"
@@ -41,12 +42,10 @@
 #include "ns3/string.h"
 #include "ns3/pointer.h"
 
-NS_LOG_COMPONENT_DEFINE ("OnOffApplication");
-
-using namespace std;
-
 namespace ns3 {
 
+NS_LOG_COMPONENT_DEFINE ("OnOffApplication");
+
 NS_OBJECT_ENSURE_REGISTERED (OnOffApplication);
 
 TypeId
@@ -87,25 +86,26 @@ OnOffApplication::GetTypeId (void)
                    MakeTypeIdAccessor (&OnOffApplication::m_tid),
                    MakeTypeIdChecker ())
     .AddTraceSource ("Tx", "A new packet is created and is sent",
-                     MakeTraceSourceAccessor (&OnOffApplication::m_txTrace))
+                     MakeTraceSourceAccessor (&OnOffApplication::m_txTrace),
+                     "ns3::Packet::TracedCallback")
   ;
   return tid;
 }
 
 
 OnOffApplication::OnOffApplication ()
+  : m_socket (0),
+    m_connected (false),
+    m_residualBits (0),
+    m_lastStartTime (Seconds (0)),
+    m_totBytes (0)
 {
-  NS_LOG_FUNCTION_NOARGS ();
-  m_socket = 0;
-  m_connected = false;
-  m_residualBits = 0;
-  m_lastStartTime = Seconds (0);
-  m_totBytes = 0;
+  NS_LOG_FUNCTION (this);
 }
 
 OnOffApplication::~OnOffApplication()
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
 }
 
 void 
@@ -134,7 +134,7 @@ OnOffApplication::AssignStreams (int64_t stream)
 void
 OnOffApplication::DoDispose (void)
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
 
   m_socket = 0;
   // chain up
@@ -144,17 +144,31 @@ OnOffApplication::DoDispose (void)
 // Application Methods
 void OnOffApplication::StartApplication () // Called at time specified by Start
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
 
   // Create the socket if not already
   if (!m_socket)
     {
       m_socket = Socket::CreateSocket (GetNode (), m_tid);
-      m_socket->Bind ();
+      if (Inet6SocketAddress::IsMatchingType (m_peer))
+        {
+          m_socket->Bind6 ();
+        }
+      else if (InetSocketAddress::IsMatchingType (m_peer) ||
+               PacketSocketAddress::IsMatchingType (m_peer))
+        {
+          m_socket->Bind ();
+        }
       m_socket->Connect (m_peer);
       m_socket->SetAllowBroadcast (true);
       m_socket->ShutdownRecv ();
+
+      m_socket->SetConnectCallback (
+        MakeCallback (&OnOffApplication::ConnectionSucceeded, this),
+        MakeCallback (&OnOffApplication::ConnectionFailed, this));
     }
+  m_cbrRateFailSafe = m_cbrRate;
+
   // Insure no pending event
   CancelEvents ();
   // If we are not yet connected, there is nothing to do here
@@ -165,7 +179,7 @@ void OnOffApplication::StartApplication () // Called at time specified by Start
 
 void OnOffApplication::StopApplication () // Called at time specified by Stop
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
 
   CancelEvents ();
   if(m_socket != 0)
@@ -180,15 +194,16 @@ void OnOffApplication::StopApplication () // Called at time specified by Stop
 
 void OnOffApplication::CancelEvents ()
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
 
-  if (m_sendEvent.IsRunning ())
+  if (m_sendEvent.IsRunning () && m_cbrRateFailSafe == m_cbrRate )
     { // Cancel the pending send packet event
       // Calculate residual bits since last packet sent
       Time delta (Simulator::Now () - m_lastStartTime);
       int64x64_t bits = delta.To (Time::S) * m_cbrRate.GetBitRate ();
       m_residualBits += bits.GetHigh ();
     }
+  m_cbrRateFailSafe = m_cbrRate;
   Simulator::Cancel (m_sendEvent);
   Simulator::Cancel (m_startStopEvent);
 }
@@ -196,7 +211,7 @@ void OnOffApplication::CancelEvents ()
 // Event handlers
 void OnOffApplication::StartSending ()
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
   m_lastStartTime = Simulator::Now ();
   ScheduleNextTx ();  // Schedule the send packet event
   ScheduleStopEvent ();
@@ -204,7 +219,7 @@ void OnOffApplication::StartSending ()
 
 void OnOffApplication::StopSending ()
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
   CancelEvents ();
 
   ScheduleStartEvent ();
@@ -213,7 +228,7 @@ void OnOffApplication::StopSending ()
 // Private helpers
 void OnOffApplication::ScheduleNextTx ()
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
 
   if (m_maxBytes == 0 || m_totBytes < m_maxBytes)
     {
@@ -233,7 +248,7 @@ void OnOffApplication::ScheduleNextTx ()
 
 void OnOffApplication::ScheduleStartEvent ()
 {  // Schedules the event to start sending data (switch to the "On" state)
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
 
   Time offInterval = Seconds (m_offTime->GetValue ());
   NS_LOG_LOGIC ("start at " << offInterval);
@@ -242,7 +257,7 @@ void OnOffApplication::ScheduleStartEvent ()
 
 void OnOffApplication::ScheduleStopEvent ()
 {  // Schedules the event to stop sending data (switch to "Off" state)
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
 
   Time onInterval = Seconds (m_onTime->GetValue ());
   NS_LOG_LOGIC ("stop at " << onInterval);
@@ -252,7 +267,7 @@ void OnOffApplication::ScheduleStopEvent ()
 
 void OnOffApplication::SendPacket ()
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
 
   NS_ASSERT (m_sendEvent.IsExpired ());
   Ptr<Packet> packet = Create<Packet> (m_pktSize);
@@ -282,18 +297,17 @@ void OnOffApplication::SendPacket ()
   ScheduleNextTx ();
 }
 
-void OnOffApplication::ConnectionSucceeded (Ptr<Socket>)
-{
-  NS_LOG_FUNCTION_NOARGS ();
 
+void OnOffApplication::ConnectionSucceeded (Ptr<Socket> socket)
+{
+  NS_LOG_FUNCTION (this << socket);
   m_connected = true;
-  ScheduleStartEvent ();
 }
 
-void OnOffApplication::ConnectionFailed (Ptr<Socket>)
+void OnOffApplication::ConnectionFailed (Ptr<Socket> socket)
 {
-  NS_LOG_FUNCTION_NOARGS ();
-  cout << "OnOffApplication, Connection Failed" << endl;
+  NS_LOG_FUNCTION (this << socket);
 }
 
+
 } // Namespace ns3
