@@ -27,14 +27,14 @@
 #include "ns3/pointer.h"
 #include <cmath>
 #include "buildings-propagation-loss-model.h"
-#include "ns3/buildings-mobility-model.h"
+#include <ns3/mobility-building-info.h>
 #include "ns3/enum.h"
 
 
-NS_LOG_COMPONENT_DEFINE ("BuildingsPropagationLossModel");
-
 namespace ns3 {
 
+NS_LOG_COMPONENT_DEFINE ("BuildingsPropagationLossModel");
+
 NS_OBJECT_ENSURE_REGISTERED (BuildingsPropagationLossModel);
 
 BuildingsPropagationLossModel::ShadowingLoss::ShadowingLoss ()
@@ -100,7 +100,7 @@ BuildingsPropagationLossModel::BuildingsPropagationLossModel ()
 }
 
 double
-BuildingsPropagationLossModel::ExternalWallLoss (Ptr<BuildingsMobilityModel> a) const
+BuildingsPropagationLossModel::ExternalWallLoss (Ptr<MobilityBuildingInfo> a) const
 {
   double loss = 0.0;
   Ptr<Building> aBuilding = a->GetBuilding ();
@@ -124,7 +124,7 @@ BuildingsPropagationLossModel::ExternalWallLoss (Ptr<BuildingsMobilityModel> a)
 }
 
 double
-BuildingsPropagationLossModel::HeightLoss (Ptr<BuildingsMobilityModel> node) const
+BuildingsPropagationLossModel::HeightLoss (Ptr<MobilityBuildingInfo> node) const
 {
   double loss = 0.0;
 
@@ -134,11 +134,11 @@ BuildingsPropagationLossModel::HeightLoss (Ptr<BuildingsMobilityModel> node) con
 }
 
 double
-BuildingsPropagationLossModel::InternalWallsLoss (Ptr<BuildingsMobilityModel> a, Ptr<BuildingsMobilityModel>b) const
+BuildingsPropagationLossModel::InternalWallsLoss (Ptr<MobilityBuildingInfo> a, Ptr<MobilityBuildingInfo>b) const
 {
   // approximate the number of internal walls with the Manhattan distance in "rooms" units
-  double dx = abs (a->GetRoomNumberX () - b->GetRoomNumberX ());
-  double dy = abs (a->GetRoomNumberY () - b->GetRoomNumberY ());    
+  double dx = std::abs (a->GetRoomNumberX () - b->GetRoomNumberX ());
+  double dy = std::abs (a->GetRoomNumberY () - b->GetRoomNumberY ());    
   return m_lossInternalWall * (dx+dy);
 }
 
@@ -148,9 +148,9 @@ double
 BuildingsPropagationLossModel::GetShadowing (Ptr<MobilityModel> a, Ptr<MobilityModel> b)
 const
 {
-    Ptr<BuildingsMobilityModel> a1 = DynamicCast<BuildingsMobilityModel> (a);
-    Ptr<BuildingsMobilityModel> b1 = DynamicCast<BuildingsMobilityModel> (b);
-    NS_ASSERT_MSG ((a1 != 0) && (b1 != 0), "BuildingsPropagationLossModel only works with BuildingsMobilityModel");
+    Ptr<MobilityBuildingInfo> a1 = a->GetObject <MobilityBuildingInfo> ();
+    Ptr<MobilityBuildingInfo> b1 = b->GetObject <MobilityBuildingInfo> ();
+    NS_ASSERT_MSG ((a1 != 0) && (b1 != 0), "BuildingsPropagationLossModel only works with MobilityBuildingInfo");
   
   std::map<Ptr<MobilityModel>,  std::map<Ptr<MobilityModel>, ShadowingLoss> >::iterator ait = m_shadowingLossMap.find (a);
   if (ait != m_shadowingLossMap.end ())
@@ -184,7 +184,7 @@ const
 
 
 double
-BuildingsPropagationLossModel::EvaluateSigma (Ptr<BuildingsMobilityModel> a, Ptr<BuildingsMobilityModel> b)
+BuildingsPropagationLossModel::EvaluateSigma (Ptr<MobilityBuildingInfo> a, Ptr<MobilityBuildingInfo> b)
 const
 {
   if (a->IsOutdoor ())
@@ -195,7 +195,7 @@ const
         }
       else
         {
-          double sigma = sqrt ((m_shadowingSigmaOutdoor * m_shadowingSigmaOutdoor) + (m_shadowingSigmaExtWalls * m_shadowingSigmaExtWalls));
+          double sigma = std::sqrt ((m_shadowingSigmaOutdoor * m_shadowingSigmaOutdoor) + (m_shadowingSigmaExtWalls * m_shadowingSigmaExtWalls));
           return (sigma);
         }
     }
@@ -206,7 +206,7 @@ const
     }
   else
     {
-      double sigma = sqrt ((m_shadowingSigmaOutdoor * m_shadowingSigmaOutdoor) + (m_shadowingSigmaExtWalls * m_shadowingSigmaExtWalls));
+      double sigma = std::sqrt ((m_shadowingSigmaOutdoor * m_shadowingSigmaOutdoor) + (m_shadowingSigmaExtWalls * m_shadowingSigmaExtWalls));
       return (sigma);
     }
 }
