@@ -18,24 +18,37 @@
  * Author: Mathieu Lacage <mathieu.lacage@sophia.inria.fr>
  */
 #include "watchdog.h"
+#include "log.h"
+
+
+/**
+ * \file
+ * \ingroup timer
+ * ns3::Watchdog timer class implementation.
+ */
 
 namespace ns3 {
 
+NS_LOG_COMPONENT_DEFINE ("Watchdog");
+
 Watchdog::Watchdog ()
   : m_impl (0),
     m_event (),
     m_end (MicroSeconds (0))
 {
+  NS_LOG_FUNCTION_NOARGS ();
 }
 
 Watchdog::~Watchdog ()
 {
+  NS_LOG_FUNCTION (this);
   delete m_impl;
 }
 
 void
 Watchdog::Ping (Time delay)
 {
+  NS_LOG_FUNCTION (this << delay);
   Time end = Simulator::Now () + delay;
   m_end = std::max (m_end, end);
   if (m_event.IsRunning ())
@@ -48,6 +61,7 @@ Watchdog::Ping (Time delay)
 void
 Watchdog::Expire (void)
 {
+  NS_LOG_FUNCTION (this);
   if (m_end == Simulator::Now ())
     {
       m_impl->Invoke ();
