@@ -33,16 +33,33 @@
 #include "ns3/trace-source-accessor.h"
 #include "ns3/pointer.h"
 #include "ns3/object-factory.h"
+#include "ns3/nstime.h"
 
-namespace ns3 {
+using namespace ns3;
 
+/**
+ * Test class for TracedValue callbacks.
+ * \see attribute_ValueClassTest
+ */
 class ValueClassTest 
 {
 public:
   ValueClassTest () {}
+  
+  /**
+   * TracedValue callback signature for ValueClassTest
+   *
+   * \param [in] oldValue original value of the traced variable
+   * \param [in] newValue new value of the traced variable
+   */
+  typedef void (* TracedValueCallback)(const ValueClassTest oldValue,
+                                       const ValueClassTest newValue);
+
 private:
   int m_v;
 };
+
+
 bool operator != (const ValueClassTest &a, const ValueClassTest &b)
 {
   return true;
@@ -55,6 +72,7 @@ std::istream & operator >> (std::istream &is, ValueClassTest &v)
 {
   return is;
 }
+
 ATTRIBUTE_HELPER_HEADER (ValueClassTest);
 ATTRIBUTE_HELPER_CPP (ValueClassTest);
 
@@ -169,11 +187,14 @@ public:
                      MakeValueClassTestAccessor (&AttributeObjectTest::m_valueSrc),
                      MakeValueClassTestChecker ())
       .AddTraceSource ("Source1", "help test",
-                       MakeTraceSourceAccessor (&AttributeObjectTest::m_intSrc1))
+                       MakeTraceSourceAccessor (&AttributeObjectTest::m_intSrc1),
+                       "ns3::TracedValue::Int8Callback")
       .AddTraceSource ("Source2", "help text",
-                       MakeTraceSourceAccessor (&AttributeObjectTest::m_cb))
+                       MakeTraceSourceAccessor (&AttributeObjectTest::m_cb),
+                       "ns3::AttributeObjectTest::NumericTracedCallback")
       .AddTraceSource ("ValueSource", "help text",
-                       MakeTraceSourceAccessor (&AttributeObjectTest::m_valueSrc))
+                       MakeTraceSourceAccessor (&AttributeObjectTest::m_valueSrc),
+                       "ns3::ValueClassTest::TracedValueCallback")
       .AddAttribute ("Pointer", "help text",
                      PointerValue (),
                      MakePointerAccessor (&AttributeObjectTest::m_ptr),
@@ -190,11 +211,27 @@ public:
                      CallbackValue (),
                      MakeCallbackAccessor (&AttributeObjectTest::m_cbValue),
                      MakeCallbackChecker ())
+      .AddAttribute ("TestTimeWithBounds", "help text",
+                     TimeValue (Seconds (-2)),
+                     MakeTimeAccessor (&AttributeObjectTest::m_timeWithBounds),
+                     MakeTimeChecker (Seconds (-5), Seconds (10)))
     ;
 
     return tid;
   }
 
+  AttributeObjectTest (void)
+  {
+    NS_UNUSED (m_boolTest);
+    NS_UNUSED (m_int16);
+    NS_UNUSED (m_int16WithBounds);
+    NS_UNUSED (m_uint8);
+    NS_UNUSED (m_float);
+    NS_UNUSED (m_enum);
+  }
+
+  virtual ~AttributeObjectTest (void) {};
+
   void AddToVector1 (void) { m_vector1.push_back (CreateObject<Derived> ()); }
   void AddToVector2 (void) { m_vector2.push_back (CreateObject<Derived> ()); }
 
@@ -234,6 +271,8 @@ private:
   Callback<void,int8_t> m_cbValue;
   TracedValue<int8_t> m_intSrc1;
   TracedValue<int8_t> m_intSrc2;
+
+  typedef void (* NumericTracedCallback) (const double, const int, const float);
   TracedCallback<double, int, float> m_cb;
   TracedValue<ValueClassTest> m_valueSrc;
   Ptr<Derived> m_ptr;
@@ -243,6 +282,7 @@ private:
   TracedValue<enum Test_e> m_enumSrc;
   TracedValue<double> m_doubleSrc;
   TracedValue<bool> m_boolSrc;
+  Time m_timeWithBounds;
 };
 
 NS_OBJECT_ENSURE_REGISTERED (AttributeObjectTest);
@@ -643,6 +683,73 @@ AttributeTestCase<EnumValue>::DoRun (void)
   NS_TEST_ASSERT_MSG_EQ (ok, true, "Error in SetAttributeFailSafe() but value changes");
 }
 
+template <> void
+AttributeTestCase<TimeValue>::DoRun (void)
+{
+  Ptr<AttributeObjectTest> p;
+  bool ok;
+
+  p = CreateObject<AttributeObjectTest> ();
+  NS_TEST_ASSERT_MSG_NE (p, 0, "Unable to CreateObject");
+
+  //
+  // Set value
+  //
+  ok = p->SetAttributeFailSafe ("TestTimeWithBounds", TimeValue (Seconds (5)));
+  NS_TEST_ASSERT_MSG_EQ (ok, true, "Could not SetAttributeFailSafe() via TimeValue to 5s");
+
+  ok = CheckGetCodePaths (p, "TestTimeWithBounds", "+5000000000.0ns", TimeValue (Seconds (5)));
+  NS_TEST_ASSERT_MSG_EQ (ok, true, "Attribute not set properly by SetAttributeFailSafe() (5s) via TimeValue");
+
+  ok = p->SetAttributeFailSafe ("TestTimeWithBounds", StringValue ("3s"));
+  NS_TEST_ASSERT_MSG_EQ (ok, true, "Could not SetAttributeFailSafe() via TimeValue to 5s");
+
+  ok = CheckGetCodePaths (p, "TestTimeWithBounds", "+3000000000.0ns", TimeValue (Seconds (3)));
+  NS_TEST_ASSERT_MSG_EQ (ok, true, "Attribute not set properly by SetAttributeFailSafe() (3s) via StringValue");
+
+  
+  //
+  // Attributes can have limits other than the intrinsic limits of the
+  // underlying data types.  These limits are specified in the Object.
+  //
+
+  //
+  // Set the Attribute at the positive limit
+  //
+  ok = p->SetAttributeFailSafe ("TestTimeWithBounds", TimeValue (Seconds (10)));
+  NS_TEST_ASSERT_MSG_EQ (ok, true, "Could not SetAttributeFailSafe() via TimeValue to 10");
+
+  ok = CheckGetCodePaths (p, "TestTimeWithBounds", "+10000000000.0ns", TimeValue (Seconds (10)));
+  NS_TEST_ASSERT_MSG_EQ (ok, true, "Attribute not set properly by SetAttributeFailSafe() (positive limit) via StringValue");
+
+  //
+  // Set the Attribute past the positive limit.
+  //
+  ok = p->SetAttributeFailSafe ("TestTimeWithBounds", TimeValue (Seconds (11)));
+  NS_TEST_ASSERT_MSG_EQ (ok, false, "Unexpectedly could SetAttributeFailSafe() via TimeValue to 11");
+
+  ok = CheckGetCodePaths (p, "TestTimeWithBounds", "+10000000000.0ns", TimeValue (Seconds (10)));
+  NS_TEST_ASSERT_MSG_EQ (ok, true, "Error in SetAttributeFailSafe() but value changes");
+
+  //
+  // Set the Attribute at the negative limit.
+  //
+  ok = p->SetAttributeFailSafe ("TestTimeWithBounds", TimeValue (Seconds (-5)));
+  NS_TEST_ASSERT_MSG_EQ (ok, true, "Could not SetAttributeFailSafe() via TimeValue to -5");
+
+  ok = CheckGetCodePaths (p, "TestTimeWithBounds", "-5000000000.0ns", TimeValue (Seconds (-5)));
+  NS_TEST_ASSERT_MSG_EQ (ok, true, "Attribute not set properly by SetAttributeFailSafe() (negative limit) via StringValue");
+
+  //
+  // Set the Attribute past the negative limit.
+  //
+  ok = p->SetAttributeFailSafe ("TestTimeWithBounds", TimeValue (Seconds (-6)));
+  NS_TEST_ASSERT_MSG_EQ (ok, false, "Unexpectedly could SetAttributeFailSafe() via TimeValue to -6");
+
+  ok = CheckGetCodePaths (p, "TestTimeWithBounds", "-5000000000.0ns", TimeValue (Seconds (-5)));
+  NS_TEST_ASSERT_MSG_EQ (ok, true, "Error in SetAttributeFailSafe() but value changes");
+}
+
 // ===========================================================================
 // Test the Attributes of type RandomVariableStream.
 // ===========================================================================
@@ -1274,21 +1381,20 @@ public:
 AttributesTestSuite::AttributesTestSuite ()
   : TestSuite ("attributes", UNIT)
 {
-  AddTestCase (new AttributeTestCase<BooleanValue> ("Check Attributes of type BooleanValue"));
-  AddTestCase (new AttributeTestCase<IntegerValue> ("Check Attributes of type IntegerValue"));
-  AddTestCase (new AttributeTestCase<UintegerValue> ("Check Attributes of type UintegerValue"));
-  AddTestCase (new AttributeTestCase<DoubleValue> ("Check Attributes of type DoubleValue"));
-  AddTestCase (new AttributeTestCase<EnumValue> ("Check Attributes of type EnumValue"));
-  AddTestCase (new RandomVariableStreamAttributeTestCase ("Check Attributes of type RandomVariableStream"));
-  AddTestCase (new ObjectVectorAttributeTestCase ("Check Attributes of type ObjectVectorValue"));
-  AddTestCase (new ObjectMapAttributeTestCase ("Check Attributes of type ObjectMapValue"));
-  AddTestCase (new IntegerTraceSourceAttributeTestCase ("Ensure TracedValue<uint8_t> can be set like IntegerValue"));
-  AddTestCase (new IntegerTraceSourceTestCase ("Ensure TracedValue<uint8_t> also works as trace source"));
-  AddTestCase (new TracedCallbackTestCase ("Ensure TracedCallback<double, int, float> works as trace source"));
-  AddTestCase (new PointerAttributeTestCase ("Check Attributes of type PointerValue"));
-  AddTestCase (new CallbackValueTestCase ("Check Attributes of type CallbackValue"));
+  AddTestCase (new AttributeTestCase<BooleanValue> ("Check Attributes of type BooleanValue"), TestCase::QUICK);
+  AddTestCase (new AttributeTestCase<IntegerValue> ("Check Attributes of type IntegerValue"), TestCase::QUICK);
+  AddTestCase (new AttributeTestCase<UintegerValue> ("Check Attributes of type UintegerValue"), TestCase::QUICK);
+  AddTestCase (new AttributeTestCase<DoubleValue> ("Check Attributes of type DoubleValue"), TestCase::QUICK);
+  AddTestCase (new AttributeTestCase<EnumValue> ("Check Attributes of type EnumValue"), TestCase::QUICK);
+  AddTestCase (new AttributeTestCase<TimeValue> ("Check Attributes of type TimeValue"), TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamAttributeTestCase ("Check Attributes of type RandomVariableStream"), TestCase::QUICK);
+  AddTestCase (new ObjectVectorAttributeTestCase ("Check Attributes of type ObjectVectorValue"), TestCase::QUICK);
+  AddTestCase (new ObjectMapAttributeTestCase ("Check Attributes of type ObjectMapValue"), TestCase::QUICK);
+  AddTestCase (new PointerAttributeTestCase ("Check Attributes of type PointerValue"), TestCase::QUICK);
+  AddTestCase (new CallbackValueTestCase ("Check Attributes of type CallbackValue"), TestCase::QUICK);
+  AddTestCase (new IntegerTraceSourceAttributeTestCase ("Ensure TracedValue<uint8_t> can be set like IntegerValue"), TestCase::QUICK);
+  AddTestCase (new IntegerTraceSourceTestCase ("Ensure TracedValue<uint8_t> also works as trace source"), TestCase::QUICK);
+  AddTestCase (new TracedCallbackTestCase ("Ensure TracedCallback<double, int, float> works as trace source"), TestCase::QUICK);
 }
 
 static AttributesTestSuite attributesTestSuite;
-
-} // namespace ns3
