@@ -22,11 +22,10 @@
  */
 
 
-#include <math.h>
 #include <gsl/gsl_cdf.h>
 #include <gsl/gsl_histogram.h>
 #include <gsl/gsl_sf_zeta.h>
-#include <time.h>
+#include <ctime>
 #include <fstream>
 #include <cmath>
 
@@ -163,6 +162,40 @@ RandomVariableStreamUniformTestCase::DoRun (void)
       NS_TEST_ASSERT_MSG_LT (value, max, "Value greater than or equal to maximum.");
     }
 
+  // Boundary checking on GetInteger; should be [min,max]; from bug 1964
+  static const uint32_t UNIFORM_INTEGER_MIN = 0;
+  static const uint32_t UNIFORM_INTEGER_MAX = 4294967295U;
+  // [0,0] should return 0
+  uint32_t intValue;
+  intValue = x->GetInteger (UNIFORM_INTEGER_MIN, UNIFORM_INTEGER_MIN);
+  NS_TEST_ASSERT_MSG_EQ (intValue, UNIFORM_INTEGER_MIN, "Uniform RV GetInteger boundary testing");
+  // [UNIFORM_INTEGER_MAX, UNIFORM_INTEGER_MAX] should return UNIFORM_INTEGER_MAX
+  intValue = x->GetInteger (UNIFORM_INTEGER_MAX, UNIFORM_INTEGER_MAX);
+  NS_TEST_ASSERT_MSG_EQ (intValue, UNIFORM_INTEGER_MAX, "Uniform RV GetInteger boundary testing");
+  // [0,1] should return mix of 0 or 1
+  intValue = 0;
+  for (int i = 0; i < 20; i++)
+    {
+      intValue += x->GetInteger (UNIFORM_INTEGER_MIN, UNIFORM_INTEGER_MIN + 1);
+    }
+  NS_TEST_ASSERT_MSG_GT (intValue, 0, "Uniform RV GetInteger boundary testing");
+  NS_TEST_ASSERT_MSG_LT (intValue, 20, "Uniform RV GetInteger boundary testing");
+  // [MAX-1,MAX] should return mix of MAX-1 or MAX
+  uint32_t count = 0;
+  for (int i = 0; i < 20; i++)
+    {
+      intValue = x->GetInteger (UNIFORM_INTEGER_MAX - 1, UNIFORM_INTEGER_MAX);
+      if (intValue == UNIFORM_INTEGER_MAX)
+        {
+          count++;
+        }
+    }
+  NS_TEST_ASSERT_MSG_GT (count, 0, "Uniform RV GetInteger boundary testing");
+  NS_TEST_ASSERT_MSG_LT (count, 20, "Uniform RV GetInteger boundary testing");
+  // multiple [0,UNIFORM_INTEGER_MAX] should return non-zero
+  intValue = x->GetInteger (UNIFORM_INTEGER_MIN, UNIFORM_INTEGER_MAX);
+  uint32_t intValue2 = x->GetInteger (UNIFORM_INTEGER_MIN, UNIFORM_INTEGER_MAX);
+  NS_TEST_ASSERT_MSG_GT (intValue + intValue2, 0, "Uniform RV GetInteger boundary testing");
 
 }
 
@@ -289,7 +322,7 @@ class RandomVariableStreamConstantTestCase : public TestCase
 {
 public:
   static const uint32_t N_MEASUREMENTS = 1000000;
-  static const double TOLERANCE = 1e-8;
+  static const double TOLERANCE;
 
   RandomVariableStreamConstantTestCase ();
   virtual ~RandomVariableStreamConstantTestCase ();
@@ -298,6 +331,8 @@ private:
   virtual void DoRun (void);
 };
 
+const double RandomVariableStreamConstantTestCase::TOLERANCE = 1e-8;
+
 RandomVariableStreamConstantTestCase::RandomVariableStreamConstantTestCase ()
   : TestCase ("Constant Random Variable Stream Generator")
 {
@@ -337,7 +372,7 @@ RandomVariableStreamConstantTestCase::DoRun (void)
 class RandomVariableStreamSequentialTestCase : public TestCase
 {
 public:
-  static const double TOLERANCE = 1e-8;
+  static const double TOLERANCE;
 
   RandomVariableStreamSequentialTestCase ();
   virtual ~RandomVariableStreamSequentialTestCase ();
@@ -346,6 +381,8 @@ private:
   virtual void DoRun (void);
 };
 
+const double RandomVariableStreamSequentialTestCase::TOLERANCE = 1e-8;
+
 RandomVariableStreamSequentialTestCase::RandomVariableStreamSequentialTestCase ()
   : TestCase ("Sequential Random Variable Stream Generator")
 {
@@ -1598,10 +1635,10 @@ RandomVariableStreamLogNormalTestCase::DoRun (void)
 
   // Test that values have approximately the right mean value.
   //
-  // XXX This test fails sometimes if the required tolerance is less
-  // than 3%, which may be because there is a bug in the
-  // implementation or that the mean of this distribution is more
-  // senstive to its parameters than the others are.
+  /// \todo This test fails sometimes if the required tolerance is less
+  /// than 3%, which may be because there is a bug in the
+  /// implementation or that the mean of this distribution is more
+  /// senstive to its parameters than the others are.
   double TOLERANCE = expectedMean * 3e-2;
   NS_TEST_ASSERT_MSG_EQ_TOL (valueMean, expectedMean, TOLERANCE, "Wrong mean value."); 
 }
@@ -1741,10 +1778,10 @@ RandomVariableStreamLogNormalAntitheticTestCase::DoRun (void)
 
   // Test that values have approximately the right mean value.
   //
-  // XXX This test fails sometimes if the required tolerance is less
-  // than 3%, which may be because there is a bug in the
-  // implementation or that the mean of this distribution is more
-  // senstive to its parameters than the others are.
+  /// \todo This test fails sometimes if the required tolerance is less
+  /// than 3%, which may be because there is a bug in the
+  /// implementation or that the mean of this distribution is more
+  /// senstive to its parameters than the others are.
   double TOLERANCE = expectedMean * 3e-2;
   NS_TEST_ASSERT_MSG_EQ_TOL (valueMean, expectedMean, TOLERANCE, "Wrong mean value."); 
 }
@@ -2591,7 +2628,7 @@ RandomVariableStreamZetaAntitheticTestCase::DoRun (void)
 class RandomVariableStreamDeterministicTestCase : public TestCase
 {
 public:
-  static const double TOLERANCE = 1e-8;
+  static const double TOLERANCE;
 
   RandomVariableStreamDeterministicTestCase ();
   virtual ~RandomVariableStreamDeterministicTestCase ();
@@ -2600,6 +2637,8 @@ private:
   virtual void DoRun (void);
 };
 
+const double RandomVariableStreamDeterministicTestCase::TOLERANCE = 1e-8;
+
 RandomVariableStreamDeterministicTestCase::RandomVariableStreamDeterministicTestCase ()
   : TestCase ("Deterministic Random Variable Stream Generator")
 {
@@ -2786,41 +2825,41 @@ public:
 RandomVariableStreamTestSuite::RandomVariableStreamTestSuite ()
   : TestSuite ("random-variable-stream-generators", UNIT)
 {
-  AddTestCase (new RandomVariableStreamUniformTestCase);
-  AddTestCase (new RandomVariableStreamUniformAntitheticTestCase);
-  AddTestCase (new RandomVariableStreamConstantTestCase);
-  AddTestCase (new RandomVariableStreamSequentialTestCase);
-  AddTestCase (new RandomVariableStreamNormalTestCase);
-  AddTestCase (new RandomVariableStreamNormalAntitheticTestCase);
-  AddTestCase (new RandomVariableStreamExponentialTestCase);
-  AddTestCase (new RandomVariableStreamExponentialAntitheticTestCase);
-  AddTestCase (new RandomVariableStreamParetoTestCase);
-  AddTestCase (new RandomVariableStreamParetoAntitheticTestCase);
-  AddTestCase (new RandomVariableStreamWeibullTestCase);
-  AddTestCase (new RandomVariableStreamWeibullAntitheticTestCase);
-  AddTestCase (new RandomVariableStreamLogNormalTestCase);
-  // XXX This test is currently disabled because it fails sometimes.
-  // A possible reason for the failure is that the antithetic code is
-  // not implemented properly for this log-normal case.
+  AddTestCase (new RandomVariableStreamUniformTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamUniformAntitheticTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamConstantTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamSequentialTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamNormalTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamNormalAntitheticTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamExponentialTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamExponentialAntitheticTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamParetoTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamParetoAntitheticTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamWeibullTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamWeibullAntitheticTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamLogNormalTestCase, TestCase::QUICK);
+  /// \todo This test is currently disabled because it fails sometimes.
+  /// A possible reason for the failure is that the antithetic code is
+  /// not implemented properly for this log-normal case.
   /*
-  AddTestCase (new RandomVariableStreamLogNormalAntitheticTestCase);
+  AddTestCase (new RandomVariableStreamLogNormalAntitheticTestCase, TestCase::QUICK);
   */
-  AddTestCase (new RandomVariableStreamGammaTestCase);
-  // XXX This test is currently disabled because it fails sometimes.
-  // A possible reason for the failure is that the antithetic code is
-  // not implemented properly for this gamma case.
+  AddTestCase (new RandomVariableStreamGammaTestCase, TestCase::QUICK);
+  /// \todo This test is currently disabled because it fails sometimes.
+  /// A possible reason for the failure is that the antithetic code is
+  /// not implemented properly for this gamma case.
   /*
-  AddTestCase (new RandomVariableStreamGammaAntitheticTestCase);
+  AddTestCase (new RandomVariableStreamGammaAntitheticTestCase, TestCase::QUICK);
   */
-  AddTestCase (new RandomVariableStreamErlangTestCase);
-  AddTestCase (new RandomVariableStreamErlangAntitheticTestCase);
-  AddTestCase (new RandomVariableStreamZipfTestCase);
-  AddTestCase (new RandomVariableStreamZipfAntitheticTestCase);
-  AddTestCase (new RandomVariableStreamZetaTestCase);
-  AddTestCase (new RandomVariableStreamZetaAntitheticTestCase);
-  AddTestCase (new RandomVariableStreamDeterministicTestCase);
-  AddTestCase (new RandomVariableStreamEmpiricalTestCase);
-  AddTestCase (new RandomVariableStreamEmpiricalAntitheticTestCase);
+  AddTestCase (new RandomVariableStreamErlangTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamErlangAntitheticTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamZipfTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamZipfAntitheticTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamZetaTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamZetaAntitheticTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamDeterministicTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamEmpiricalTestCase, TestCase::QUICK);
+  AddTestCase (new RandomVariableStreamEmpiricalAntitheticTestCase, TestCase::QUICK);
 }
 
 static RandomVariableStreamTestSuite randomVariableStreamTestSuite;
