@@ -21,7 +21,7 @@
 #ifndef CSMA_NET_DEVICE_H
 #define CSMA_NET_DEVICE_H
 
-#include <string.h>
+#include <cstring>
 #include "ns3/node.h"
 #include "ns3/backoff.h"
 #include "ns3/address.h"
@@ -41,7 +41,7 @@ class CsmaChannel;
 class ErrorModel;
 
 /** 
- * \defgroup csma CsmaNetDevice
+ * \defgroup csma CSMA Network Device
  *
  * This section documents the API of the ns-3 csma module. For a generic functional description, please refer to the ns-3 manual.
  */
@@ -225,7 +225,7 @@ public:
    * \brief Make and return a MAC multicast address using the provided
    *        multicast group
    *
-   * RFC 1112 says that an Ipv4 host group address is mapped to an Ethernet 
+   * \RFC{1112} says that an Ipv4 host group address is mapped to an Ethernet 
    * multicast address by placing the low-order 23-bits of the IP address into 
    * the low-order 23 bits of the Ethernet multicast address 
    * 01-00-5E-00-00-00 (hex).
