@@ -32,7 +32,6 @@
 
 #include "dsdv-routing-protocol.h"
 #include "ns3/log.h"
-#include "ns3/random-variable.h"
 #include "ns3/inet-socket-address.h"
 #include "ns3/trace-source-accessor.h"
 #include "ns3/udp-socket-factory.h"
@@ -41,10 +40,12 @@
 #include "ns3/double.h"
 #include "ns3/uinteger.h"
 
-NS_LOG_COMPONENT_DEFINE ("DsdvRoutingProtocol");
-
 namespace ns3 {
+
+NS_LOG_COMPONENT_DEFINE ("DsdvRoutingProtocol");
+  
 namespace dsdv {
+  
 NS_OBJECT_ENSURE_REGISTERED (RoutingProtocol);
 
 /// UDP Port for DSDV control traffic
@@ -188,12 +189,21 @@ RoutingProtocol::GetEnableRAFlag () const
   return EnableRouteAggregation;
 }
 
+int64_t
+RoutingProtocol::AssignStreams (int64_t stream)
+{
+  NS_LOG_FUNCTION (this << stream);
+  m_uniformRandomVariable->SetStream (stream);
+  return 1;
+}
+
 RoutingProtocol::RoutingProtocol ()
   : m_routingTable (),
     m_advRoutingTable (),
     m_queue (),
     m_periodicUpdateTimer (Timer::CANCEL_ON_DESTROY)
 {
+  m_uniformRandomVariable = CreateObject<UniformRandomVariable> ();
 }
 
 RoutingProtocol::~RoutingProtocol ()
@@ -231,7 +241,7 @@ RoutingProtocol::Start ()
   m_scb = MakeCallback (&RoutingProtocol::Send,this);
   m_ecb = MakeCallback (&RoutingProtocol::Drop,this);
   m_periodicUpdateTimer.SetFunction (&RoutingProtocol::SendPeriodicUpdate,this);
-  m_periodicUpdateTimer.Schedule (MicroSeconds (UniformVariable ().GetInteger (0,1000)));
+  m_periodicUpdateTimer.Schedule (MicroSeconds (m_uniformRandomVariable->GetInteger (0,1000)));
 }
 
 Ptr<Ipv4Route>
@@ -270,7 +280,7 @@ RoutingProtocol::RouteOutput (Ptr<Packet> p,
     }
   if (!removedAddresses.empty ())
     {
-      Simulator::Schedule (MicroSeconds (UniformVariable ().GetInteger (0,1000)),&RoutingProtocol::SendTriggeredUpdate,this);
+      Simulator::Schedule (MicroSeconds (m_uniformRandomVariable->GetInteger (0,1000)),&RoutingProtocol::SendTriggeredUpdate,this);
     }
   if (m_routingTable.LookupRoute (dst,rt))
     {
@@ -683,7 +693,7 @@ RoutingProtocol::RecvDsdv (Ptr<Socket> socket)
                       /*Received update with same seq number but with same or greater hop count.
                        * Discard that update.
                        */
-                      if (not m_advRoutingTable.AnyRunningEvent (dsdvHeader.GetDst ()))
+                      if (!m_advRoutingTable.AnyRunningEvent (dsdvHeader.GetDst ()))
                         {
                           /*update the timer only if nexthop address matches thus discarding
                            * updates to that destination from other nodes.
@@ -703,7 +713,7 @@ RoutingProtocol::RecvDsdv (Ptr<Socket> socket)
               else
                 {
                   // Received update with an old sequence number. Discard the update
-                  if (not m_advRoutingTable.AnyRunningEvent (dsdvHeader.GetDst ()))
+                  if (!m_advRoutingTable.AnyRunningEvent (dsdvHeader.GetDst ()))
                     {
                       m_advRoutingTable.DeleteRoute (dsdvHeader.GetDst ());
                     }
@@ -735,7 +745,7 @@ RoutingProtocol::RecvDsdv (Ptr<Socket> socket)
                 }
               else
                 {
-                  if (not m_advRoutingTable.AnyRunningEvent (dsdvHeader.GetDst ()))
+                  if (!m_advRoutingTable.AnyRunningEvent (dsdvHeader.GetDst ()))
                     {
                       m_advRoutingTable.DeleteRoute (dsdvHeader.GetDst ());
                     }
@@ -754,7 +764,7 @@ RoutingProtocol::RecvDsdv (Ptr<Socket> socket)
     }
   else
     {
-      Simulator::Schedule (MicroSeconds (UniformVariable ().GetInteger (0,1000)),&RoutingProtocol::SendTriggeredUpdate,this);
+      Simulator::Schedule (MicroSeconds (m_uniformRandomVariable->GetInteger (0,1000)),&RoutingProtocol::SendTriggeredUpdate,this);
     }
 }
 
@@ -778,7 +788,7 @@ RoutingProtocol::SendTriggeredUpdate ()
                                         << " SeqNo:" << i->second.GetSeqNo () << " HopCount:"
                                         << i->second.GetHop () + 1);
           RoutingTableEntry temp = i->second;
-          if ((i->second.GetEntriesChanged () == true) && (not m_advRoutingTable.AnyRunningEvent (temp.GetDestination ())))
+          if ((i->second.GetEntriesChanged () == true) && (!m_advRoutingTable.AnyRunningEvent (temp.GetDestination ())))
             {
               dsdvHeader.SetDst (i->second.GetDestination ());
               dsdvHeader.SetDstSeqno (i->second.GetSeqNo ());
@@ -904,7 +914,7 @@ RoutingProtocol::SendPeriodicUpdate ()
       socket->SendTo (packet, 0, InetSocketAddress (destination, DSDV_PORT));
       NS_LOG_FUNCTION ("PeriodicUpdate Packet UID is : " << packet->GetUid ());
     }
-  m_periodicUpdateTimer.Schedule (m_periodicUpdateInterval + MicroSeconds (25 * UniformVariable ().GetInteger (0,1000)));
+  m_periodicUpdateTimer.Schedule (m_periodicUpdateInterval + MicroSeconds (25 * m_uniformRandomVariable->GetInteger (0,1000)));
 }
 
 void
@@ -947,8 +957,8 @@ RoutingProtocol::NotifyInterfaceUp (uint32_t i)
   Ptr<Socket> socket = Socket::CreateSocket (GetObject<Node> (),UdpSocketFactory::GetTypeId ());
   NS_ASSERT (socket != 0);
   socket->SetRecvCallback (MakeCallback (&RoutingProtocol::RecvDsdv,this));
-  socket->BindToNetDevice (l3->GetNetDevice (i));
   socket->Bind (InetSocketAddress (Ipv4Address::GetAny (), DSDV_PORT));
+  socket->BindToNetDevice (l3->GetNetDevice (i));
   socket->SetAllowBroadcast (true);
   socket->SetAttribute ("IpTtl",UintegerValue (1));
   m_socketAddresses.insert (std::make_pair (socket,iface));
@@ -1004,9 +1014,9 @@ RoutingProtocol::NotifyAddAddress (uint32_t i,
       Ptr<Socket> socket = Socket::CreateSocket (GetObject<Node> (),UdpSocketFactory::GetTypeId ());
       NS_ASSERT (socket != 0);
       socket->SetRecvCallback (MakeCallback (&RoutingProtocol::RecvDsdv,this));
-      socket->BindToNetDevice (l3->GetNetDevice (i));
       // Bind to any IP address so that broadcasts can be received
       socket->Bind (InetSocketAddress (Ipv4Address::GetAny (), DSDV_PORT));
+      socket->BindToNetDevice (l3->GetNetDevice (i));
       socket->SetAllowBroadcast (true);
       m_socketAddresses.insert (std::make_pair (socket,iface));
       Ptr<NetDevice> dev = m_ipv4->GetNetDevice (m_ipv4->GetInterfaceForAddress (iface.GetLocal ()));
@@ -1138,7 +1148,7 @@ RoutingProtocol::SendPacketFromQueue (Ipv4Address dst,
       ucb (route,p,header);
       if (m_queue.GetSize () != 0 && m_queue.Find (dst))
         {
-          Simulator::Schedule (MilliSeconds (UniformVariable ().GetInteger (0,100)),
+          Simulator::Schedule (MilliSeconds (m_uniformRandomVariable->GetInteger (0,100)),
                                &RoutingProtocol::SendPacketFromQueue,this,dst,route);
         }
     }
@@ -1181,7 +1191,7 @@ RoutingProtocol::MergeTriggerPeriodicUpdates ()
       for (std::map<Ipv4Address, RoutingTableEntry>::const_iterator i = allRoutes.begin (); i != allRoutes.end (); ++i)
         {
           RoutingTableEntry advEntry = i->second;
-          if ((advEntry.GetEntriesChanged () == true) && (not m_advRoutingTable.AnyRunningEvent (advEntry.GetDestination ())))
+          if ((advEntry.GetEntriesChanged () == true) && (!m_advRoutingTable.AnyRunningEvent (advEntry.GetDestination ())))
             {
               if (!(advEntry.GetSeqNo () % 2))
                 {
