@@ -33,7 +33,7 @@
 #include "ns3/string.h"
 #include "ns3/yans-wifi-helper.h"
 #include "ns3/nqos-wifi-mac-helper.h"
-#include <math.h>
+#include <cmath>
 
 using namespace ns3;
 
@@ -740,6 +740,7 @@ BatteryLifetimeTest::ConstantLoadTest (double load, Time expLifetime)
   // Set alpha & beta values
   rvModelHelper.Set ("RvBatteryModelAlphaValue", DoubleValue (m_alpha));
   rvModelHelper.Set ("RvBatteryModelBetaValue", DoubleValue (m_beta));
+  rvModelHelper.Set ("RvBatteryModelLowBatteryThreshold", DoubleValue (0.0));
   // install source
   EnergySourceContainer sources = rvModelHelper.Install (c);
   // device energy model
@@ -827,6 +828,7 @@ BatteryLifetimeTest::VariableLoadTest (std::vector<double> loads,
   // Set alpha & beta values
   rvModelHelper.Set ("RvBatteryModelAlphaValue", DoubleValue (m_alpha));
   rvModelHelper.Set ("RvBatteryModelBetaValue", DoubleValue (m_beta));
+  rvModelHelper.Set ("RvBatteryModelLowBatteryThreshold", DoubleValue (0.0));
   // install source
   EnergySourceContainer sources = rvModelHelper.Install (c);
   // device energy model
@@ -883,7 +885,7 @@ public:
 RvBatteryModelTestSuite::RvBatteryModelTestSuite ()
   : TestSuite ("rv-battery-model", SYSTEM)
 {
-  AddTestCase (new BatteryLifetimeTest);
+  AddTestCase (new BatteryLifetimeTest, TestCase::QUICK);
 }
 
 // create an instance of the test suite
