@@ -19,9 +19,12 @@
  */
 #include "ipv4-raw-socket-factory.h"
 #include "ns3/uinteger.h"
+#include "ns3/log.h"
 
 namespace ns3 {
 
+NS_LOG_COMPONENT_DEFINE ("Ipv4RawSocketFactory");
+
 NS_OBJECT_ENSURE_REGISTERED (Ipv4RawSocketFactory);
 
 TypeId Ipv4RawSocketFactory::GetTypeId (void)
