@@ -16,6 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Author: Jaume Nin <jnin@cttc.es>
+ *         Nicola Baldo <nbaldo@cttc.es>
  */
 
 #include "radio-bearer-stats-calculator.h"
@@ -25,8 +26,7 @@
 #include <vector>
 #include <algorithm>
 
-namespace ns3
-{
+namespace ns3 {
 
 NS_LOG_COMPONENT_DEFINE ("RadioBearerStatsCalculator");
 
@@ -59,8 +59,16 @@ RadioBearerStatsCalculator::GetTypeId (void)
   static TypeId tid =
     TypeId ("ns3::RadioBearerStatsCalculator")
     .SetParent<LteStatsCalculator> ().AddConstructor<RadioBearerStatsCalculator> ()
-    .AddAttribute ("StartTime", "Start time of the on going epoch.", TimeValue (Seconds (0.)),MakeTimeAccessor (&RadioBearerStatsCalculator::m_startTime), MakeTimeChecker ())
-    .AddAttribute ("EpochDuration", "Epoch duration.", TimeValue (Seconds (0.25)), MakeTimeAccessor (&RadioBearerStatsCalculator::m_epochDuration), MakeTimeChecker ())
+    .AddAttribute ("StartTime", "Start time of the on going epoch.", 
+                   TimeValue (Seconds (0.)),
+                   MakeTimeAccessor (&RadioBearerStatsCalculator::SetStartTime,
+                                     &RadioBearerStatsCalculator::GetStartTime), 
+                   MakeTimeChecker ())
+    .AddAttribute ("EpochDuration", "Epoch duration.", 
+                   TimeValue (Seconds (0.25)), 
+                   MakeTimeAccessor (&RadioBearerStatsCalculator::GetEpoch,
+                                     &RadioBearerStatsCalculator::SetEpoch), 
+                   MakeTimeChecker ())
     .AddAttribute ("DlRlcOutputFilename",
                    "Name of the file where the downlink results will be saved.",
                    StringValue ("DlRlcStats.txt"),
@@ -95,14 +103,40 @@ RadioBearerStatsCalculator::DoDispose ()
     }
 }
 
-void
-RadioBearerStatsCalculator::UlTxPdu (uint64_t imsi, uint16_t rnti, uint8_t lcid, uint32_t packetSize)
+void 
+RadioBearerStatsCalculator::SetStartTime (Time t)
 {
-  NS_LOG_FUNCTION (this << "UlTxPDU" << imsi << rnti << (uint32_t) lcid << packetSize);
-  CheckEpoch ();
+  m_startTime = t;
+  RescheduleEndEpoch ();
+}
+
+Time 
+RadioBearerStatsCalculator::GetStartTime () const
+{
+  return m_startTime;
+}
+
+void 
+RadioBearerStatsCalculator::SetEpoch (Time e)
+{
+  m_epochDuration = e;
+  RescheduleEndEpoch ();
+}
+
+Time 
+RadioBearerStatsCalculator::GetEpoch () const
+{
+  return m_epochDuration;  
+}
+
+void
+RadioBearerStatsCalculator::UlTxPdu (uint16_t cellId, uint64_t imsi, uint16_t rnti, uint8_t lcid, uint32_t packetSize)
+{
+  NS_LOG_FUNCTION (this << "UlTxPDU" << cellId << imsi << rnti << (uint32_t) lcid << packetSize);
   ImsiLcidPair_t p (imsi, lcid);
-  if (Simulator::Now () > m_startTime)
+  if (Simulator::Now () >= m_startTime)
     {
+      m_ulCellId[p] = cellId;
       m_flowId[p] = LteFlowId_t (rnti, lcid);
       m_ulTxPackets[p]++;
       m_ulTxData[p] += packetSize;
@@ -113,10 +147,9 @@ RadioBearerStatsCalculator::UlTxPdu (uint64_t imsi, uint16_t rnti, uint8_t lcid,
 void
 RadioBearerStatsCalculator::DlTxPdu (uint16_t cellId, uint64_t imsi, uint16_t rnti, uint8_t lcid, uint32_t packetSize)
 {
-  NS_LOG_FUNCTION (this << "DlTxPDU" << imsi << rnti << (uint32_t) lcid << packetSize);
-  CheckEpoch ();
+  NS_LOG_FUNCTION (this << "DlTxPDU" << cellId << imsi << rnti << (uint32_t) lcid << packetSize);
   ImsiLcidPair_t p (imsi, lcid);
-  if (Simulator::Now () > m_startTime)
+  if (Simulator::Now () >= m_startTime)
     {
       m_dlCellId[p] = cellId;
       m_flowId[p] = LteFlowId_t (rnti, lcid);
@@ -130,11 +163,9 @@ void
 RadioBearerStatsCalculator::UlRxPdu (uint16_t cellId, uint64_t imsi, uint16_t rnti, uint8_t lcid, uint32_t packetSize,
                                      uint64_t delay)
 {
-  NS_LOG_FUNCTION (this << "UlRxPDU" << imsi << rnti << (uint32_t) lcid << packetSize << delay);
+  NS_LOG_FUNCTION (this << "UlRxPDU" << cellId << imsi << rnti << (uint32_t) lcid << packetSize << delay);
   ImsiLcidPair_t p (imsi, lcid);
-  CheckEpoch ();
-
-  if (Simulator::Now () > m_startTime)
+  if (Simulator::Now () >= m_startTime)
     {
       m_ulCellId[p] = cellId;
       m_ulRxPackets[p]++;
@@ -154,13 +185,13 @@ RadioBearerStatsCalculator::UlRxPdu (uint16_t cellId, uint64_t imsi, uint16_t rn
 }
 
 void
-RadioBearerStatsCalculator::DlRxPdu (uint64_t imsi, uint16_t rnti, uint8_t lcid, uint32_t packetSize, uint64_t delay)
+RadioBearerStatsCalculator::DlRxPdu (uint16_t cellId, uint64_t imsi, uint16_t rnti, uint8_t lcid, uint32_t packetSize, uint64_t delay)
 {
-  NS_LOG_FUNCTION (this << "DlRxPDU" << imsi << rnti << (uint32_t) lcid << packetSize << delay);
-  CheckEpoch ();
+  NS_LOG_FUNCTION (this << "DlRxPDU" << cellId << imsi << rnti << (uint32_t) lcid << packetSize << delay);
   ImsiLcidPair_t p (imsi, lcid);
-  if (Simulator::Now () > m_startTime)
+  if (Simulator::Now () >= m_startTime)
     {
+      m_dlCellId[p] = cellId;
       m_dlRxPackets[p]++;
       m_dlRxData[p] += packetSize;
 
@@ -347,26 +378,22 @@ RadioBearerStatsCalculator::ResetResults (void)
 }
 
 void
-RadioBearerStatsCalculator::CheckEpoch (void)
+RadioBearerStatsCalculator::RescheduleEndEpoch (void)
 {
   NS_LOG_FUNCTION (this);
-
-  if (Simulator::Now () > m_startTime + m_epochDuration)
-    {
-      ShowResults ();
-      ResetResults ();
-      StartEpoch ();
-    }
+  m_endEpochEvent.Cancel ();
+  NS_ASSERT (Simulator::Now ().GetMilliSeconds () == 0); // below event time assumes this
+  m_endEpochEvent = Simulator::Schedule (m_startTime + m_epochDuration, &RadioBearerStatsCalculator::EndEpoch, this);
 }
 
 void
-RadioBearerStatsCalculator::StartEpoch (void)
+RadioBearerStatsCalculator::EndEpoch (void)
 {
   NS_LOG_FUNCTION (this);
-  while (Simulator::Now () > m_startTime + m_epochDuration)
-    {
-      m_startTime += m_epochDuration;
-    }
+  ShowResults ();
+  ResetResults ();
+  m_startTime += m_epochDuration;
+  m_endEpochEvent = Simulator::Schedule (m_epochDuration, &RadioBearerStatsCalculator::EndEpoch, this);
 }
 
 uint32_t
