@@ -21,14 +21,23 @@
 
 #include "ff-mac-scheduler.h"
 #include <ns3/log.h>
+#include <ns3/enum.h>
 
-NS_LOG_COMPONENT_DEFINE ("FfMacScheduler");
 
 namespace ns3 {
 
+NS_LOG_COMPONENT_DEFINE ("FfMacScheduler");
+
 NS_OBJECT_ENSURE_REGISTERED (FfMacScheduler);
 
 
+FfMacScheduler::FfMacScheduler ()
+: m_ulCqiFilter (ALL_UL_CQI)
+{
+  NS_LOG_FUNCTION (this);
+}
+
+
 FfMacScheduler::~FfMacScheduler ()
 {
   NS_LOG_FUNCTION (this);
@@ -44,10 +53,19 @@ TypeId
 FfMacScheduler::GetTypeId (void)
 {
   static TypeId tid = TypeId ("ns3::FfMacScheduler")
-    .SetParent<Object> ();
+    .SetParent<Object> ()
+    .AddAttribute ("UlCqiFilter",
+                   "The filter to apply on UL CQIs received",
+                   EnumValue (FfMacScheduler::ALL_UL_CQI),
+                   MakeEnumAccessor (&FfMacScheduler::m_ulCqiFilter),
+                   MakeEnumChecker (FfMacScheduler::SRS_UL_CQI, "SRS_UL_CQI",
+                                    FfMacScheduler::PUSCH_UL_CQI, "PUSCH_UL_CQI",
+                                    FfMacScheduler::ALL_UL_CQI, "ALL_UL_CQI"))
+    ;
   return tid;
 }
 
+
 } // namespace ns3
 
 
