@@ -5,7 +5,7 @@ close all;
 %% Table 22.7 Reference sensitivity.
 
 f = 2160e6;  % carrier freq Hz, EARFCN = 500 (downlink)
-nrbs = 25; % tx bandwdith configuration in number of RBs
+nrbs = 25; % tx bandwidth configuration in number of RBs
 bw = nrbs * 180000; % bandwidth in Hz, note that this is smaller than
 		    % the nominal Channel Bandwdith, see TS 36.101 fig 5.6-1
 kT = -174; % noise PSD in dBm / Hz
