@@ -36,12 +36,14 @@
 #include "ns3/trace-source-accessor.h"
 #include "ie-dot11s-perr.h"
 
-NS_LOG_COMPONENT_DEFINE ("HwmpProtocol");
-
 namespace ns3 {
+
+NS_LOG_COMPONENT_DEFINE ("HwmpProtocol");
+  
 namespace dot11s {
 
 NS_OBJECT_ENSURE_REGISTERED (HwmpProtocol);
+  
 TypeId
 HwmpProtocol::GetTypeId ()
 {
@@ -162,7 +164,8 @@ HwmpProtocol::GetTypeId ()
     .AddTraceSource ( "RouteDiscoveryTime",
                       "The time of route discovery procedure",
                       MakeTraceSourceAccessor (
-                        &HwmpProtocol::m_routeDiscoveryTimeCallback)
+                        &HwmpProtocol::m_routeDiscoveryTimeCallback),
+                      "ns3::Time::TracedCallback"
                       )
   ;
   return tid;
@@ -201,7 +204,7 @@ HwmpProtocol::~HwmpProtocol ()
 }
 
 void
-HwmpProtocol::DoStart ()
+HwmpProtocol::DoInitialize ()
 {
   m_coefficient->SetAttribute ("Max", DoubleValue (m_randomStart.GetSeconds ()));
   if (m_isRoot)
