@@ -33,9 +33,12 @@
 #include "ns3/wifi-net-device.h"
 #include "ns3/trace-source-accessor.h"
 
-NS_LOG_COMPONENT_DEFINE ("PeerManagementProtocol");
 namespace ns3 {
+
+NS_LOG_COMPONENT_DEFINE ("PeerManagementProtocol");
+  
 namespace dot11s {
+  
 /***************************************************
  * PeerManager
  ***************************************************/
@@ -72,11 +75,13 @@ PeerManagementProtocol::GetTypeId (void)
                     )
     .AddTraceSource ("LinkOpen",
                      "New peer link opened",
-                     MakeTraceSourceAccessor (&PeerManagementProtocol::m_linkOpenTraceSrc)
+                     MakeTraceSourceAccessor (&PeerManagementProtocol::m_linkOpenTraceSrc),
+                     "ns3::PeerManagementProtocol::LinkOpenCloseTracedCallback"
                      )
     .AddTraceSource ("LinkClose",
                      "New peer link closed",
-                     MakeTraceSourceAccessor (&PeerManagementProtocol::m_linkCloseTraceSrc)
+                     MakeTraceSourceAccessor (&PeerManagementProtocol::m_linkCloseTraceSrc),
+                     "ns3::PeerManagementProtocol::LinkOpenCloseTracedCallback"
                      )
 
   ;
@@ -587,7 +592,7 @@ PeerManagementProtocol::AssignStreams (int64_t stream)
 }
 
 void
-PeerManagementProtocol::DoStart ()
+PeerManagementProtocol::DoInitialize ()
 {
   // If beacon interval is equal to the neighbor's one and one o more beacons received
   // by my neighbor coincide with my beacon - apply random uniformly distributed shift from
