@@ -49,15 +49,30 @@
 #include "ns3/config.h"
 #include "ns3/ns2-mobility-helper.h"
 
-namespace ns3 {
+using namespace ns3;
+
+NS_LOG_COMPONENT_DEFINE ("ns2-mobility-helper-test-suite");
 
 // -----------------------------------------------------------------------------
 // Testing
 // -----------------------------------------------------------------------------
-bool operator== (Vector const & a, Vector const & b)
+bool AreVectorsEqual (Vector const & actual, Vector const & limit, double tol)
 {
-  return (a.x == b.x && a.y == b.y && a.z == b.z);
+  if (actual.x > limit.x + tol || actual.x < limit.x - tol)
+    {
+      return false;
+    }
+  if (actual.y > limit.y + tol || actual.y < limit.y - tol)
+    {
+      return false;
+    }
+  if (actual.z > limit.z + tol || actual.z < limit.z - tol)
+    {
+      return false;
+    }
+  return true;
 }
+
 /**
  * Every test case is supposed to:
  *  1. Generate short mobility trace file
@@ -172,8 +187,9 @@ private:
         Ptr<MobilityModel> mob = node->GetObject<MobilityModel> ();
         NS_TEST_ASSERT_MSG_NE_RETURNS_BOOL (mob, 0, "Can't find mobility for node " << rp.node);
 
-        NS_TEST_EXPECT_MSG_EQ (rp.pos, mob->GetPosition (), "Initial position mismatch for node " << rp.node);
-        NS_TEST_EXPECT_MSG_EQ (rp.vel, mob->GetVelocity (), "Initial velocity mismatch for node " << rp.node);
+        double tol = 0.001;
+        NS_TEST_EXPECT_MSG_EQ (AreVectorsEqual (mob->GetPosition (), rp.pos, tol), true, "Initial position mismatch for node " << rp.node);
+        NS_TEST_EXPECT_MSG_EQ (AreVectorsEqual (mob->GetVelocity (), rp.vel, tol), true, "Initial velocity mismatch for node " << rp.node);
 
         m_nextRefPoint++;
       }
@@ -199,8 +215,10 @@ private:
     ReferencePoint const & ref = m_reference [m_nextRefPoint++];
     NS_TEST_EXPECT_MSG_EQ (time, ref.time, "Time mismatch");
     NS_TEST_EXPECT_MSG_EQ (id, ref.node, "Node ID mismatch at time " << time.GetSeconds () << " s");
-    NS_TEST_EXPECT_MSG_EQ (pos, ref.pos, "Position mismatch at time " << time.GetSeconds () << " s for node " << id);
-    NS_TEST_EXPECT_MSG_EQ (vel, ref.vel, "Velocity mismatch at time " << time.GetSeconds () << " s for node " << id);
+
+    double tol = 0.001;
+    NS_TEST_EXPECT_MSG_EQ (AreVectorsEqual (pos, ref.pos, tol), true, "Position mismatch at time " << time.GetSeconds () << " s for node " << id);
+    NS_TEST_EXPECT_MSG_EQ (AreVectorsEqual (vel, ref.vel, tol), true, "Velocity mismatch at time " << time.GetSeconds () << " s for node " << id);
   }
 
   void DoSetup ()
@@ -211,7 +229,6 @@ private:
   void DoTeardown ()
   {
     Names::Clear ();
-    std::remove (m_traceFile.c_str ());
     Simulator::Destroy ();
   }
 
@@ -257,7 +274,7 @@ public:
                  "$node_(0) set Z_ 3.0\n"
                  );
     t->AddReferencePoint ("0", 0, Vector (1, 2, 3), Vector (0, 0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
 
     // Check parsing comments, empty lines and no EOF at the end of file
     t = new Ns2MobilityHelperTest ("comments", Seconds (1));
@@ -269,7 +286,7 @@ public:
                  "#$node_(0) set Z_ 100 #"
                  );
     t->AddReferencePoint ("0", 0, Vector (1, 2, 3), Vector (0, 0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
 
     // Simple setdest. Arguments are interpreted as x, y, speed by default
     t = new Ns2MobilityHelperTest ("simple setdest", Seconds (10));
@@ -278,7 +295,7 @@ public:
     t->AddReferencePoint ("0", 0, Vector (0, 0, 0), Vector (0, 0, 0));
     t->AddReferencePoint ("0", 1, Vector (0, 0, 0), Vector (5, 0, 0));
     t->AddReferencePoint ("0", 6, Vector (25, 0, 0), Vector (0, 0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
 
     // Several set and setdest. Arguments are interpreted as x, y, speed by default
     t = new Ns2MobilityHelperTest ("square setdest", Seconds (6));
@@ -299,7 +316,7 @@ public:
     t->AddReferencePoint ("0", 4, Vector (0, 5, 0), Vector (0, 0, 0));
     t->AddReferencePoint ("0", 4, Vector (0, 5, 0), Vector (0, -5, 0));
     t->AddReferencePoint ("0", 5, Vector (0, 0, 0), Vector (0,  0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
 
     // Copy of previous test case but with the initial positions at
     // the end of the trace rather than at the beginning.
@@ -323,7 +340,7 @@ public:
     t->AddReferencePoint ("0", 4, Vector (10, 15, 0), Vector (0, 0, 0));
     t->AddReferencePoint ("0", 4, Vector (10, 15, 0), Vector (0, -5, 0));
     t->AddReferencePoint ("0", 5, Vector (10, 10, 0), Vector (0,  0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
 
     // Scheduled set position
     t = new Ns2MobilityHelperTest ("scheduled set position", Seconds (2));
@@ -334,7 +351,7 @@ public:
     t->AddReferencePoint ("0", 1, Vector (10, 0, 0), Vector (0, 0, 0));
     t->AddReferencePoint ("0", 1, Vector (10, 0, 10), Vector (0, 0, 0));
     t->AddReferencePoint ("0", 1, Vector (10, 10, 10), Vector (0, 0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
 
     // Malformed lines
     t = new Ns2MobilityHelperTest ("malformed lines", Seconds (2));
@@ -350,7 +367,7 @@ public:
     t->AddReferencePoint ("0", 0, Vector (1, 2, 3), Vector (0, 0, 0));
     t->AddReferencePoint ("0", 1, Vector (1, 2, 3), Vector (1, 0, 0));
     t->AddReferencePoint ("0", 2, Vector (2, 2, 3), Vector (0, 0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
 
     // Non possible values
     t = new Ns2MobilityHelperTest ("non possible values", Seconds (2));
@@ -370,7 +387,7 @@ public:
     t->AddReferencePoint ("0", 0, Vector (1, 2, 3), Vector (0, 0, 0));
     t->AddReferencePoint ("0", 1, Vector (1, 2, 3), Vector (1, 0, 0));
     t->AddReferencePoint ("0", 2, Vector (2, 2, 3), Vector (0, 0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
 
     // More than one node
     t = new Ns2MobilityHelperTest ("few nodes, combinations of set and setdest", Seconds (10), 3);
@@ -398,7 +415,7 @@ public:
     t->AddReferencePoint ("2", 4, Vector (0, 5, 0), Vector (0, 0, 0));
     t->AddReferencePoint ("2", 4, Vector (0, 5, 0), Vector (0, -5, 0));
     t->AddReferencePoint ("2", 5, Vector (0, 0, 0), Vector (0,  0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
 
     // Test for Speed == 0, that acts as stop the node.
     t = new Ns2MobilityHelperTest ("setdest with speed cero", Seconds (10));
@@ -409,7 +426,7 @@ public:
     t->AddReferencePoint ("0", 1, Vector (0, 0, 0), Vector (5, 0, 0));
     t->AddReferencePoint ("0", 6, Vector (25, 0, 0), Vector (0, 0, 0));
     t->AddReferencePoint ("0", 7, Vector (25, 0, 0), Vector (0, 0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
 
 
     // Test negative positions
@@ -424,7 +441,7 @@ public:
     t->AddReferencePoint ("0", 2, Vector (0, 0, 0), Vector (0, 0, 0));
     t->AddReferencePoint ("0", 2, Vector (0, 0, 0), Vector (0, -1, 0));
     t->AddReferencePoint ("0", 3, Vector (0, -1, 0), Vector (0, 0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
 
     // Sqare setdest with values in the form 1.0e+2
     t = new Ns2MobilityHelperTest ("Foalt numbers in 1.0e+2 format", Seconds (6));
@@ -444,7 +461,7 @@ public:
     t->AddReferencePoint ("0", 4, Vector (0, 100, 0), Vector (0, 0, 0));
     t->AddReferencePoint ("0", 4, Vector (0, 100, 0), Vector (0, -100, 0));
     t->AddReferencePoint ("0", 5, Vector (0, 0, 0), Vector (0,  0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
     t = new Ns2MobilityHelperTest ("Bug 1219 testcase", Seconds (16));
     t->SetTrace ("$node_(0) set X_ 0.0\n"
                  "$node_(0) set Y_ 0.0\n"
@@ -456,14 +473,14 @@ public:
     t->AddReferencePoint ("0", 1, Vector (0, 0, 0), Vector (0,  1, 0));
     t->AddReferencePoint ("0", 6, Vector (0, 5, 0), Vector (0,  -1, 0));
     t->AddReferencePoint ("0", 16, Vector (0, -10, 0), Vector (0, 0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
     t = new Ns2MobilityHelperTest ("Bug 1059 testcase", Seconds (16));
     t->SetTrace ("$node_(0) set X_ 10.0\r\n"
                  "$node_(0) set Y_ 0.0\r\n"
                  );
     //                     id  t  position         velocity
     t->AddReferencePoint ("0", 0, Vector (10, 0, 0), Vector (0,  0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
     t = new Ns2MobilityHelperTest ("Bug 1301 testcase", Seconds (16));
     t->SetTrace ("$node_(0) set X_ 10.0\n"
                  "$node_(0) set Y_ 0.0\n"
@@ -473,11 +490,31 @@ public:
     // Moving to the current position must change nothing. No NaN
     // speed must be.
     t->AddReferencePoint ("0", 0, Vector (10, 0, 0), Vector (0,  0, 0));
-    AddTestCase (t);
+    AddTestCase (t, TestCase::QUICK);
 
+    t = new Ns2MobilityHelperTest ("Bug 1316 testcase", Seconds (1000));
+    t->SetTrace ("$node_(0) set X_ 350.00000000000000\n"
+                 "$node_(0) set Y_ 50.00000000000000\n"
+                 "$ns_ at 50.00000000000000  \"$node_(0) setdest 400.00000000000000 50.00000000000000 1.00000000000000\"\n"
+                 "$ns_ at 150.00000000000000 \"$node_(0) setdest 400.00000000000000 150.00000000000000 4.00000000000000\"\n"
+                 "$ns_ at 300.00000000000000 \"$node_(0) setdest 250.00000000000000 150.00000000000000 3.00000000000000\"\n"
+                 "$ns_ at 350.00000000000000 \"$node_(0) setdest 250.00000000000000 50.00000000000000 1.00000000000000\"\n"
+                 "$ns_ at 600.00000000000000 \"$node_(0) setdest 250.00000000000000 1050.00000000000000 2.00000000000000\"\n"
+                 "$ns_ at 900.00000000000000 \"$node_(0) setdest 300.00000000000000 650.00000000000000 2.50000000000000\"\n"
+                 );
+    t->AddReferencePoint ("0", 0.000, Vector (350.000, 50.000, 0.000), Vector (0.000, 0.000, 0.000));
+    t->AddReferencePoint ("0", 50.000, Vector (350.000, 50.000, 0.000), Vector (1.000, 0.000, 0.000));
+    t->AddReferencePoint ("0", 100.000, Vector (400.000, 50.000, 0.000), Vector (0.000, 0.000, 0.000));
+    t->AddReferencePoint ("0", 150.000, Vector (400.000, 50.000, 0.000), Vector (0.000, 4.000, 0.000));
+    t->AddReferencePoint ("0", 175.000, Vector (400.000, 150.000, 0.000), Vector (0.000, 0.000, 0.000));
+    t->AddReferencePoint ("0", 300.000, Vector (400.000, 150.000, 0.000), Vector (-3.000, 0.000, 0.000));
+    t->AddReferencePoint ("0", 350.000, Vector (250.000, 150.000, 0.000), Vector (0.000, 0.000, 0.000));
+    t->AddReferencePoint ("0", 350.000, Vector (250.000, 150.000, 0.000), Vector (0.000, -1.000, 0.000));
+    t->AddReferencePoint ("0", 450.000, Vector (250.000,  50.000, 0.000), Vector (0.000, 0.000, 0.000));
+    t->AddReferencePoint ("0", 600.000, Vector (250.000,  50.000, 0.000), Vector (0.000, 2.000, 0.000));
+    t->AddReferencePoint ("0", 900.000, Vector (250.000,  650.000, 0.000), Vector (2.500, 0.000, 0.000));
+    t->AddReferencePoint ("0", 920.000, Vector (300.000,  650.000, 0.000), Vector (0.000, 0.000, 0.000));
+    AddTestCase (t, TestCase::QUICK);
 
   }
 } g_ns2TransmobilityHelperTestSuite;
-
-
-} // namespace ns3
