@@ -26,6 +26,8 @@
 #include "ns3/mobility-module.h"
 #include "ns3/internet-module.h"
 #include "ns3/netanim-module.h"
+#include "ns3/basic-energy-source.h"
+#include "ns3/simple-device-energy-model.h"
 
 
 
@@ -115,6 +117,17 @@ main (int argc, char *argv[])
   AnimationInterface::SetConstantPosition (p2pNodes.Get (1), 10, 30); 
   AnimationInterface::SetConstantPosition (csmaNodes.Get (1), 10, 33); 
 
+  Ptr<BasicEnergySource> energySource = CreateObject<BasicEnergySource>();
+  Ptr<SimpleDeviceEnergyModel> energyModel = CreateObject<SimpleDeviceEnergyModel>();
+
+  energySource->SetInitialEnergy (300);
+  energyModel->SetEnergySource (energySource);
+  energySource->AppendDeviceEnergyModel (energyModel);
+  energyModel->SetCurrentA (20);
+
+  // aggregate energy source to node
+  wifiApNode.Get (0)->AggregateObject (energySource);
+
   // Install internet stack
 
   InternetStackHelper stack;
@@ -151,14 +164,28 @@ main (int argc, char *argv[])
 
   Ipv4GlobalRoutingHelper::PopulateRoutingTables ();
   Simulator::Stop (Seconds (15.0));
-  AnimationInterface::SetNodeDescription (wifiApNode, "AP"); // Optional
-  AnimationInterface::SetNodeDescription (wifiStaNodes, "STA"); // Optional
-  AnimationInterface::SetNodeDescription (csmaNodes, "CSMA"); // Optional
-  AnimationInterface::SetNodeColor (wifiApNode, 0, 255, 0); // Optional
-  AnimationInterface::SetNodeColor (wifiStaNodes, 255, 0, 0); // Optional
-  AnimationInterface::SetNodeColor (csmaNodes, 0, 0, 255); // Optional
+
   AnimationInterface anim ("wireless-animation.xml"); // Mandatory
-  anim.EnablePacketMetadata (true); // Optional
+  for (uint32_t i = 0; i < wifiStaNodes.GetN (); ++i)
+    {
+      anim.UpdateNodeDescription (wifiStaNodes.Get (i), "STA"); // Optional
+      anim.UpdateNodeColor (wifiStaNodes.Get (i), 255, 0, 0); // Optional
+    }
+  for (uint32_t i = 0; i < wifiApNode.GetN (); ++i)
+    {
+      anim.UpdateNodeDescription (wifiApNode.Get (i), "AP"); // Optional
+      anim.UpdateNodeColor (wifiApNode.Get (i), 0, 255, 0); // Optional
+    }
+  for (uint32_t i = 0; i < csmaNodes.GetN (); ++i)
+    {
+      anim.UpdateNodeDescription (csmaNodes.Get (i), "CSMA"); // Optional
+      anim.UpdateNodeColor (csmaNodes.Get (i), 0, 0, 255); // Optional 
+    }
+
+  anim.EnablePacketMetadata (); // Optional
+  anim.EnableIpv4RouteTracking ("routingtable-wireless.xml", Seconds (0), Seconds (5), Seconds (0.25)); //Optional
+  anim.EnableWifiMacCounters (Seconds (0), Seconds (10)); //Optional
+  anim.EnableWifiPhyCounters (Seconds (0), Seconds (10)); //Optional
   Simulator::Run ();
   Simulator::Destroy ();
   return 0;
