@@ -22,12 +22,12 @@
 #include "ns3/log.h"
 #include "ns3/simulator.h"
 #include <string>
-#include <stdarg.h>
-
-NS_LOG_COMPONENT_DEFINE ("Packet");
+#include <cstdarg>
 
 namespace ns3 {
 
+NS_LOG_COMPONENT_DEFINE ("Packet");
+
 uint32_t Packet::m_globalUid = 0;
 
 TypeId 
@@ -112,7 +112,9 @@ void
 PacketTagIterator::Item::GetTag (Tag &tag) const
 {
   NS_ASSERT (tag.GetInstanceTypeId () == m_data->tid);
-  tag.Deserialize (TagBuffer ((uint8_t*)m_data->data, (uint8_t*)m_data->data+PACKET_TAG_MAX_SIZE));
+  tag.Deserialize (TagBuffer ((uint8_t*)m_data->data,
+                              (uint8_t*)m_data->data
+                              + PacketTagList::TagData::MAX_SIZE));
 }
 
 
@@ -362,19 +364,6 @@ Packet::RemoveAllByteTags (void)
   m_byteTagList.RemoveAll ();
 }
 
-uint8_t const *
-Packet::PeekData (void) const
-{
-  NS_LOG_FUNCTION (this);
-  uint32_t oldStart = m_buffer.GetCurrentStartOffset ();
-  uint8_t const * data = m_buffer.PeekData ();
-  uint32_t newStart = m_buffer.GetCurrentStartOffset ();
- 
-  // Update tag offsets if buffer offsets were changed
-  const_cast<ByteTagList &>(m_byteTagList).AddAtStart (newStart - oldStart, newStart);
-  return data;
-}
-
 uint32_t 
 Packet::CopyData (uint8_t *buffer, uint32_t size) const
 {
@@ -423,6 +412,14 @@ Packet::PrintByteTags (std::ostream &os) const
     }
 }
 
+std::string
+Packet::ToString() const
+{
+  std::ostringstream oss;
+  Print (oss);
+  return oss.str();
+}
+
 void 
 Packet::Print (std::ostream &os) const
 {
@@ -582,7 +579,7 @@ uint32_t Packet::GetSerializedSize (void) const
     }
 
   //Tag size
-  //XXX
+  /// \todo Serialze Tags size
   //size += m_tags.GetSerializedSize ();
 
   // increment total size by size of meta-data 
@@ -656,7 +653,7 @@ Packet::Serialize (uint8_t* buffer, uint32_t maxSize) const
     }
 
   // Serialize Tags
-  // XXX
+  /// \todo Serialize Tags
 
   // Serialize Metadata
   uint32_t metaSize = m_metadata.GetSerializedSize ();
@@ -754,7 +751,7 @@ Packet::Deserialize (const uint8_t* buffer, uint32_t size)
     }
 
   // read tags
-  //XXX
+  /// \todo Deserialize Tags
   //uint32_t tagsDeserialized = m_tags.Deserialize (buffer.Begin ());
   //buffer.RemoveAtStart (tagsDeserialized);
 
@@ -841,6 +838,7 @@ Packet::AddPacketTag (const Tag &tag) const
   NS_LOG_FUNCTION (this << tag.GetInstanceTypeId ().GetName () << tag.GetSerializedSize ());
   m_packetTagList.Add (tag);
 }
+
 bool 
 Packet::RemovePacketTag (Tag &tag)
 {
@@ -848,6 +846,14 @@ Packet::RemovePacketTag (Tag &tag)
   bool found = m_packetTagList.Remove (tag);
   return found;
 }
+bool
+Packet::ReplacePacketTag (Tag &tag)
+{
+  NS_LOG_FUNCTION (this << tag.GetInstanceTypeId ().GetName () << tag.GetSerializedSize ());
+  bool found = m_packetTagList.Replace (tag);
+  return found;
+}
+
 bool 
 Packet::PeekPacketTag (Tag &tag) const
 {
