@@ -74,7 +74,7 @@ namespace ns3 {
 class TraceContainer;
 class UniformRandomVariable;
 
-/*
+/**
  * \ingroup queue
  *
  * \brief A RED packet queue
@@ -82,47 +82,46 @@ class UniformRandomVariable;
 class RedQueue : public Queue
 {
 public:
+  /**
+   * \brief Get the type ID.
+   * \return the object TypeId
+   */
   static TypeId GetTypeId (void);
-  /*
+  /**
    * \brief RedQueue Constructor
    *
    * Create a RED queue
    */
   RedQueue ();
 
-  /*
+  /**
    * \brief Destructor
    *
    * Destructor
    */ 
   virtual ~RedQueue ();
 
-  /*
+  /**
    * \brief Stats
-   *
    */
   typedef struct
-  {
-    // Early probability drops
-    uint32_t unforcedDrop;
-    // Forced drops, qavg > max threshold
-    uint32_t forcedDrop;
-    // Drops due to queue limits
-    uint32_t qLimDrop;
+  {   
+    uint32_t unforcedDrop;  //!< Early probability drops
+    uint32_t forcedDrop;    //!< Forced drops, qavg > max threshold
+    uint32_t qLimDrop;      //!< Drops due to queue limits
   } Stats;
 
-  /* 
+  /** 
    * \brief Drop types
-   *
    */
   enum
   {
-    DTYPE_NONE,        // Ok, no drop
-    DTYPE_FORCED,      // A "forced" drop
-    DTYPE_UNFORCED,    // An "unforced" (random) drop
+    DTYPE_NONE,        //!< Ok, no drop
+    DTYPE_FORCED,      //!< A "forced" drop
+    DTYPE_UNFORCED,    //!< An "unforced" (random) drop
   };
 
-  /*
+  /**
    * \brief Set the operating mode of this queue.
    *  Set operating mode
    *
@@ -130,7 +129,7 @@ public:
    */
   void SetMode (RedQueue::QueueMode mode);
 
-  /*
+  /**
    * \brief Get the encapsulation mode of this queue.
    * Get the encapsulation mode of this queue
    *
@@ -138,29 +137,29 @@ public:
    */
   RedQueue::QueueMode GetMode (void);
 
-  /*
+  /**
    * \brief Get the current value of the queue in bytes or packets.
    *
    * \returns The queue size in bytes or packets.
    */
   uint32_t GetQueueSize (void);
 
-  /*
+  /**
    * \brief Set the limit of the queue.
    *
    * \param lim The limit in bytes or packets.
    */
   void SetQueueLimit (uint32_t lim);
 
-  /*
+  /**
    * \brief Set the thresh limits of RED.
    *
-   * \param min Minimum thresh in bytes or packets.
-   * \param max Maximum thresh in bytes or packets.
+   * \param minTh Minimum thresh in bytes or packets.
+   * \param maxTh Maximum thresh in bytes or packets.
    */
   void SetTh (double minTh, double maxTh);
 
-  /*
+  /**
    * \brief Get the RED statistics after running.
    *
    * \returns The drop statistics.
@@ -182,90 +181,103 @@ private:
   virtual Ptr<Packet> DoDequeue (void);
   virtual Ptr<const Packet> DoPeek (void) const;
 
-  // ...
+  /**
+   * \brief Initialize the queue parameters.
+   *
+   * Note: if the link bandwidth changes in the course of the
+   * simulation, the bandwidth-dependent RED parameters do not change.
+   * This should be fixed, but it would require some extra parameters,
+   * and didn't seem worth the trouble...
+   */
   void InitializeParams (void);
-  // Compute the average queue size
+  /**
+   * \brief Compute the average queue size
+   * \param nQueued number of queued packets
+   * \param m simulated number of packets arrival during idle period
+   * \param qAvg average queue size
+   * \param qW queue weight given to cur q size sample
+   * \returns new average queue size
+   */
   double Estimator (uint32_t nQueued, uint32_t m, double qAvg, double qW);
-  // Check if packet p needs to be dropped due to probability mark
+  /**
+   * \brief Check if packet p needs to be dropped due to probability mark
+   * \param p packet
+   * \param qSize queue size
+   * \returns 0 for no drop/mark, 1 for drop
+   */
   uint32_t DropEarly (Ptr<Packet> p, uint32_t qSize);
-  // Returns a probability using these function parameters for the DropEarly funtion
-  double CalculatePNew (double qAvg, double maxTh, bool gentle, double vA,
+  /**
+   * \brief Returns a probability using these function parameters for the DropEarly function
+   * \param qAvg Average queue length
+   * \param maxTh Max avg length threshold
+   * \param gentle "gentle" algorithm
+   * \param vA vA
+   * \param vB vB
+   * \param vC vC
+   * \param vD vD
+   * \param maxP max_p
+   * \returns Prob. of packet drop before "count"
+   */
+  double CalculatePNew (double qAvg, double , bool gentle, double vA,
                         double vB, double vC, double vD, double maxP);
-  // Returns a probability using these function parameters for the DropEarly funtion
+  /**
+   * \brief Returns a probability using these function parameters for the DropEarly function
+   * \param p Prob. of packet drop before "count"
+   * \param count number of packets since last random number generation
+   * \param countBytes number of bytes since last drop
+   * \param meanPktSize Avg pkt size
+   * \param wait True for waiting between dropped packets
+   * \param size packet size
+   * \returns Prob. of packet drop
+   */
   double ModifyP (double p, uint32_t count, uint32_t countBytes,
                   uint32_t meanPktSize, bool wait, uint32_t size);
 
-  std::list<Ptr<Packet> > m_packets;
+  std::list<Ptr<Packet> > m_packets; //!< packets in the queue
 
-  uint32_t m_bytesInQueue;
-  bool m_hasRedStarted;
-  Stats m_stats;
+  uint32_t m_bytesInQueue; //!< bytes in the queue
+  bool m_hasRedStarted; //!< True if RED has started
+  Stats m_stats; //!< RED statistics
 
   // ** Variables supplied by user
-  // Bytes or packets?
-  QueueMode m_mode;
-  // Avg pkt size
-  uint32_t m_meanPktSize;
-  // Avg pkt size used during idle times
-  uint32_t m_idlePktSize;
-  // True for waiting between dropped packets
-  bool m_isWait;
-  // True to increases dropping prob. slowly when ave queue exceeds maxthresh
-  bool m_isGentle;
-  // Min avg length threshold (bytes)
-  double m_minTh;
-  // Max avg length threshold (bytes), should be >= 2*minTh
-  double m_maxTh;
-  // Queue limit in bytes / packets
-  uint32_t m_queueLimit;
-  // Queue weight given to cur q size sample
-  double m_qW;
-  // The max probability of dropping a packet
-  double m_lInterm;
-  // Ns-1 compatibility
-  bool m_isNs1Compat;
-  // Link bandwidth
-  DataRate m_linkBandwidth;
-  // Link delay
-  Time m_linkDelay;
+  QueueMode m_mode;         //!< Mode (Bytes or packets)
+  uint32_t m_meanPktSize;   //!< Avg pkt size
+  uint32_t m_idlePktSize;   //!< Avg pkt size used during idle times
+  bool m_isWait;            //!< True for waiting between dropped packets
+  bool m_isGentle;          //!< True to increases dropping prob. slowly when ave queue exceeds maxthresh
+  double m_minTh;           //!< Min avg length threshold (bytes)
+  double m_maxTh;           //!< Max avg length threshold (bytes), should be >= 2*minTh
+  uint32_t m_queueLimit;    //!< Queue limit in bytes / packets
+  double m_qW;              //!< Queue weight given to cur queue size sample
+  double m_lInterm;         //!< The max probability of dropping a packet
+  bool m_isNs1Compat;       //!< Ns-1 compatibility
+  DataRate m_linkBandwidth; //!< Link bandwidth
+  Time m_linkDelay;         //!< Link delay
 
   // ** Variables maintained by RED
-  // Prob. of packet drop before "count"
-  double m_vProb1;
-  // v_prob = v_a * v_ave + v_b
-  double m_vA;
-  double m_vB;
-  // Used for "gentle" mode
-  double m_vC;
-  // Used for "gentle" mode
-  double m_vD;
-  // Current max_p
-  double m_curMaxP;
-  // Prob. of packet drop
-  double m_vProb;
-  // # of bytes since last drop
-  uint32_t m_countBytes;
-  // 0 when average queue first exceeds thresh
-  uint32_t m_old;
-  // 0/1 idle status
-  uint32_t m_idle;
-  // packet time constant in packets/second
-  double m_ptc;
-  // Average queue length
-  double m_qAvg;
-  // number of packets since last random number generation
-  uint32_t m_count;
-  /*
+  double m_vProb1;          //!< Prob. of packet drop before "count"
+  double m_vA;              //!< 1.0 / (m_maxTh - m_minTh)
+  double m_vB;              //!< -m_minTh / (m_maxTh - m_minTh)
+  double m_vC;              //!< (1.0 - m_curMaxP) / m_maxTh - used in "gentle" mode
+  double m_vD;              //!< 2.0 * m_curMaxP - 1.0 - used in "gentle" mode
+  double m_curMaxP;         //!< Current max_p
+  double m_vProb;           //!< Prob. of packet drop
+  uint32_t m_countBytes;    //!< Number of bytes since last drop
+  uint32_t m_old;           //!< 0 when average queue first exceeds threshold
+  uint32_t m_idle;          //!< 0/1 idle status
+  double m_ptc;             //!< packet time constant in packets/second
+  double m_qAvg;            //!< Average queue length
+  uint32_t m_count;         //!< Number of packets since last random number generation
+  /**
    * 0 for default RED
    * 1 experimental (see red-queue.cc)
    * 2 experimental (see red-queue.cc)
    * 3 use Idle packet size in the ptc
    */
   uint32_t m_cautious;
-  // Start of current idle period
-  Time m_idleTime;
+  Time m_idleTime;          //!< Start of current idle period
 
-  Ptr<UniformRandomVariable> m_uv;
+  Ptr<UniformRandomVariable> m_uv;  //!< rng stream
 };
 
 }; // namespace ns3
