@@ -26,10 +26,10 @@
 #include "propagation-loss-model.h"
 #include "jakes-propagation-loss-model.h"
 
-NS_LOG_COMPONENT_DEFINE ("JakesProcess");
-
 namespace ns3 {
 
+NS_LOG_COMPONENT_DEFINE ("JakesProcess");
+
 /// Represents a single oscillator
 JakesProcess::Oscillator::Oscillator (std::complex<double> amplitude, double initialPhase, double omega) :
   m_amplitude (amplitude),
@@ -40,7 +40,7 @@ JakesProcess::Oscillator::Oscillator (std::complex<double> amplitude, double ini
 std::complex<double>
 JakesProcess::Oscillator::GetValueAt (Time at) const
 {
-  return (m_amplitude * cos (at.GetSeconds () * m_omega + m_phase));
+  return (m_amplitude * std::cos (at.GetSeconds () * m_omega + m_phase));
 }
 
 NS_OBJECT_ENSURE_REGISTERED (JakesProcess);
@@ -85,7 +85,7 @@ JakesProcess::SetNOscillators (unsigned int nOscillators)
 void
 JakesProcess::SetDopplerFrequencyHz (double dopplerFrequencyHz)
 {
-  m_omegaDopplerMax = 2 * dopplerFrequencyHz * JakesPropagationLossModel::PI;
+  m_omegaDopplerMax = 2 * dopplerFrequencyHz * M_PI;
 }
 
 void
@@ -101,12 +101,12 @@ JakesProcess::ConstructOscillators ()
       unsigned int n = i + 1;
       /// 1. Rotation speed
       /// 1a. Initiate \f[ \alpha_n = \frac{2\pi n - \pi + \theta}{4M},  n=1,2, \ldots,M\f], n is oscillatorNumber, M is m_nOscillators
-      double alpha = (2.0 * JakesPropagationLossModel::PI * n - JakesPropagationLossModel::PI + theta) / (4.0 * m_nOscillators);
+      double alpha = (2.0 * M_PI * n - M_PI + theta) / (4.0 * m_nOscillators);
       /// 1b. Initiate rotation speed:
-      double omega = m_omegaDopplerMax * cos (alpha);
+      double omega = m_omegaDopplerMax * std::cos (alpha);
       /// 2. Initiate complex amplitude:
       double psi = m_jakes->GetUniformRandomVariable ()->GetValue ();
-      std::complex<double> amplitude = std::complex<double> (cos (psi), sin (psi)) * 2.0 / sqrt (m_nOscillators);
+      std::complex<double> amplitude = std::complex<double> (std::cos (psi), std::sin (psi)) * 2.0 / std::sqrt (m_nOscillators);
       /// 3. Construct oscillator:
       m_oscillators.push_back (Oscillator (amplitude, phi, omega)); 
     }
@@ -144,7 +144,7 @@ double
 JakesProcess::GetChannelGainDb () const
 {
   std::complex<double> complexGain = GetComplexGain ();
-  return (10 * log10 ((pow (complexGain.real (), 2) + pow (complexGain.imag (), 2)) / 2));
+  return (10 * std::log10 ((std::pow (complexGain.real (), 2) + std::pow (complexGain.imag (), 2)) / 2));
 }
 
 } // namespace ns3
