@@ -42,10 +42,10 @@
 #include "ss-link-manager.h"
 #include "bandwidth-manager.h"
 
-NS_LOG_COMPONENT_DEFINE ("SubscriberStationNetDevice");
-
 namespace ns3 {
 
+NS_LOG_COMPONENT_DEFINE ("SubscriberStationNetDevice");
+
 NS_OBJECT_ENSURE_REGISTERED (SubscriberStationNetDevice);
 
 Time SubscriberStationNetDevice::GetDefaultLostDlMapInterval ()
@@ -183,22 +183,27 @@ SubscriberStationNetDevice::GetTypeId (void)
 
     .AddTraceSource ("SSTxDrop",
                      "A packet has been dropped in the MAC layer before being queued for transmission.",
-                     MakeTraceSourceAccessor (&SubscriberStationNetDevice::m_ssTxDropTrace))
+                     MakeTraceSourceAccessor (&SubscriberStationNetDevice::m_ssTxDropTrace),
+                     "ns3::Packet::TracedCallback")
 
     .AddTraceSource ("SSPromiscRx",
                      "A packet has been received by this device, has been passed up from the physical layer "
                      "and is being forwarded up the local protocol stack.  This is a promiscuous trace,",
-                     MakeTraceSourceAccessor (&SubscriberStationNetDevice::m_ssPromiscRxTrace))
+                     MakeTraceSourceAccessor (&SubscriberStationNetDevice::m_ssPromiscRxTrace),
+                     "ns3::Packet::TracedCallback")
 
     .AddTraceSource ("SSRx",
                      "A packet has been received by this device, has been passed up from the physical layer "
                      "and is being forwarded up the local protocol stack.  This is a non-promiscuous trace,",
-                     MakeTraceSourceAccessor (&SubscriberStationNetDevice::m_ssRxTrace))
+                     MakeTraceSourceAccessor (&SubscriberStationNetDevice::m_ssRxTrace),
+                     "ns3::Packet::TracedCallback")
 
     .AddTraceSource ("SSRxDrop",
                      "A packet has been dropped in the MAC layer after it has been passed up from the physical "
                      "layer.",
-                     MakeTraceSourceAccessor (&SubscriberStationNetDevice::m_ssRxDropTrace));
+                     MakeTraceSourceAccessor (&SubscriberStationNetDevice::m_ssRxDropTrace),
+                     "ns3::Packet::TracedCallback")
+    ;
   return tid;
 }
 
@@ -1230,7 +1235,7 @@ SubscriberStationNetDevice::ProcessUcd (const Ucd &ucd)
 
     }
   SetCurrentUcd (ucd);
-  m_linkManager->SetRangingCW ((uint8_t) pow ((double) 2, (double) ucd.GetRangingBackoffStart ()) - 1); // initializing ranging CW
+  m_linkManager->SetRangingCW ((uint8_t) std::pow ((double) 2, (double) ucd.GetRangingBackoffStart ()) - 1); // initializing ranging CW
   OfdmUcdChannelEncodings ucdChnlEncodings = ucd.GetChannelEncodings ();
 
   std::vector<OfdmUlBurstProfile> ulBurstProfiles = ucd.GetUlBurstProfiles ();
