@@ -10,6 +10,8 @@
 #include <map>
 #include <set>
 #include <exception>
+#include <algorithm>
+#include <stdexcept>
 
 #include <stdint.h>
 
@@ -85,6 +87,11 @@ inline std::ostream & operator << (std::ostream &os, Foo const &foo)
     return os;
 }
 
+inline bool is_unique(const Foo& foo)
+{
+    return foo.instance_count == 1;
+}
+
 class Zoo
 // -#- automatic_type_narrowing=True -#-
 {
@@ -161,7 +168,7 @@ public:
     virtual int get_int (int x) {
         return x;
     }
-    
+
     static int instance_count;
 
     virtual ~Zbr () {
@@ -183,7 +190,7 @@ class Foobar
 {
     Foobar (const Foobar &);
     Foobar& operator= (const Foobar &);
-    
+
 public:
     static int instance_count;
 
@@ -198,7 +205,7 @@ class SomeObject
 {
 public:
     std::string m_prefix;
-    
+
     enum {
         TYPE_FOO,
         TYPE_BAR,
@@ -365,7 +372,7 @@ public:
     const Foo * get_foo_shared_ptr () {
         return m_foo_shared_ptr;
     }
-    
+
     // -#- @return(caller_owns_return=true) -#-
     Foo * get_foo_ptr () {
         Foo *foo = m_foo_ptr;
@@ -513,7 +520,7 @@ namespace xpto
 
     std::string get_foo_datum(FooXpto const &foo);
 
-    struct XptoClass 
+    struct XptoClass
     {
         SomeClass* GetSomeClass()
         // -#- @return(caller_owns_return=true) -#-
@@ -757,7 +764,7 @@ namespace TopNs
     };
 
     namespace PrefixBottomNs {
-        class PrefixInner : public OuterBase 
+        class PrefixInner : public OuterBase
         {
         public:
             PrefixInner () {}
@@ -807,7 +814,7 @@ int set_simple_list (SimpleStructList list);
 class TestContainer
 {
 public:
-    
+
     std::set<float> m_floatSet;
 
     TestContainer () : m_vec (NULL) {
@@ -845,7 +852,7 @@ private:
     SimpleStructMap m_simpleMap;
 
     std::vector<std::string> *m_vec;
-    
+
 };
 
 std::map<std::string, int> get_map ();
@@ -974,7 +981,7 @@ private:
 
     // disable the copy constructor
     ManipulatedObject (const ManipulatedObject &ctor_arg);
-    
+
 public:
     ManipulatedObject () : m_value (0) {}
     void SetValue (int value) { m_value = value; }
@@ -985,7 +992,7 @@ public:
 class ReferenceManipulator
 {
     ManipulatedObject m_obj;
-    
+
 public:
     ReferenceManipulator () {}
     virtual ~ReferenceManipulator () {}
@@ -1004,25 +1011,117 @@ public:
 class VectorLike
 {
     std::vector<double> m_vec;
-    
+
 public:
-    VectorLike () 
+    VectorLike ()
         {
         }
     // -#- name=__len__ -#-
-    std::vector<double>::size_type get_len () const 
+    std::vector<double>::size_type get_len () const
         {
             return m_vec.size ();
         }
-    // -#- name=__setitem__ -#-
-    void set_item (std::vector<double>::size_type index, double value)
+    // -#- name=__add__-#-
+    VectorLike add_VectorLike(const VectorLike& rhs)
         {
+            VectorLike result(*this);
+            std::copy(rhs.m_vec.begin(), rhs.m_vec.end(), std::back_inserter(result.m_vec));
+            return result;
+        }
+    // -#- name=__iadd__-#-
+    VectorLike& iadd_VectorLike(const VectorLike& rhs)
+        {
+            std::copy(rhs.m_vec.begin(), rhs.m_vec.end(), std::back_inserter(m_vec));
+            return *this;
+        }
+    // -#- name=__mul__-#-
+    VectorLike mul_VectorLike(const unsigned n)
+        {
+            VectorLike result;
+            if (n > 0)
+            {
+                result.m_vec.reserve(n * m_vec.size());
+                for (unsigned i = 0; i != n; ++i)
+                {
+                    std::copy(m_vec.begin(), m_vec.end(), std::back_inserter(result.m_vec));
+                }
+            }
+            return result;
+        }
+    // -#- name=__imul__-#-
+    VectorLike& imul_VectorLike(const unsigned n)
+        {
+            if (n > 0)
+            {
+                const unsigned n0 = m_vec.size();
+                m_vec.reserve(n * n0);
+                for (unsigned i = 0; i < n - 1; ++i)
+                {
+                    std::copy(m_vec.begin(), m_vec.begin() + n0, std::back_inserter(m_vec));
+                }
+            }
+            else
+            {
+                *this = VectorLike();
+            }
+            return *this;
+        }
+    // -#- name=__setitem__ -#-
+    int set_item (int index, double value)
+        {
+            const int n = this->get_len();
+            index = (index < 0 ? (n + 1 + index) : index);
+            if (index >= n) return -1;
             m_vec[index] = value;
+            return 0;
         }
     // -#- name=__getitem__ -#-
-    double get_item (std::vector<double>::size_type index) const
+    double get_item (int index) const
         {
-            return m_vec[index];
+            const int n = this->get_len();
+            index = (index < 0 ? (n + 1 + index) : index);
+            try
+            {
+                return m_vec.at(index);
+            }
+            catch (std::out_of_range)
+            {
+                PyErr_SetString(PyExc_IndexError, "Container index out of range");
+                return 0.0;
+            }
+        }
+    // -#- name=__setslice__ -#-
+    int set_slice (int index1, int index2, const VectorLike& values)
+        {
+            const int n = this->get_len();
+            index1 = (index1 < 0 ? (n + 1 + index1) : index1);
+            index2 = (index2 < 0 ? (n + 1 + index2) : index2);
+            if (index1 >= n or index2 > n) return -1;
+            for (int i = index1; i < index2; ++i)
+            {
+                if (this->set_item(i, values.get_item(i - index1)) != 0) return -1;
+            }
+            return 0;
+        }
+    // -#- name=__getslice__ -#-
+    VectorLike get_slice (int index1, int index2) const
+        {
+            VectorLike result;
+            const int n = this->get_len();
+            index1 = (index1 < 0 ? (n + 1 + index1) : index1);
+            index2 = (index2 < 0 ? (n + 1 + index2) : index2);
+            for (int i = index1; i < index2; ++i)
+            {
+                result.m_vec.push_back(this->get_item(i));
+            }
+            return result;
+        }
+    // -#- name=__contains__ -#-
+    int contains_value (double value) const
+        {
+            return ((std::find(m_vec.begin(), m_vec.end(), value) == m_vec.end()) ?
+                    0 :
+                    1);
         }
 
     void append (double value)
@@ -1040,9 +1139,9 @@ public:
 class VectorLike2
 {
     std::vector<double> m_vec;
-    
+
 public:
-    VectorLike2 () 
+    VectorLike2 ()
         {
         }
 
@@ -1060,7 +1159,7 @@ public:
 class MapLike
 {
     std::map<int, double> m_map;
-    
+
 public:
     MapLike () {}
 
@@ -1081,7 +1180,7 @@ struct Error
 struct DomainError : public Error
 // -#- exception -#-
 {
-};    
+};
 
 
 // returns 1/x, raises DomainError if x == 0
@@ -1089,24 +1188,28 @@ double my_inverse_func (double x) throw (DomainError);
 double my_inverse_func2 (double x) throw (std::exception);
 
 // the following function throws an exception but forgets to declare it
-// -#- throw=std::exception -#- 
+// -#- throw=std::exception -#-
 double my_inverse_func3 (double x);
 
 class ClassThatThrows
 {
 public:
     ClassThatThrows (double x) throw (DomainError);
-    
+
     // returns 1/x, raises DomainError if x == 0
     double my_inverse_method (double x) throw (DomainError);
     double my_inverse_method2 (double x) throw (std::exception);
 
     // the following method throws an exception but forgets to declare it
-    // -#- throw=std::exception -#- 
+    // -#- throw=std::exception -#-
     double my_inverse_method3 (double x);
 
+    // -#- throw=std::out_of_range -#-
+    int throw_out_of_range() throw (std::out_of_range);
+
     virtual int throw_error () const throw (std::exception);
 
+    virtual ~ClassThatThrows () {}
 };
 
 
@@ -1128,7 +1231,7 @@ class property
 {
 private:
     ValueType m_value;
-    
+
 public:
     property<ValueType> ()
     {
@@ -1171,13 +1274,13 @@ public:
 
     // -#- @return(reference_existing_object=true) -#-
     const Foobar* getFoobarInternalPtr () { return &m_foobar; }
-    
+
     // -#- @return(reference_existing_object=true) -#-
     Foobar& getFoobarInternalRef () { return m_foobar; }
 
     // -#- @return(return_internal_reference=true) -#-
     Foobar* getFoobarInternalPtr2 () { return &m_foobar; }
-    
+
     // -#- @return(return_internal_reference=true) -#-
     Foobar& getFoobarInternalRef2 () { return m_foobar; }
 
@@ -1189,7 +1292,7 @@ class MIRoot
 {
 public:
     MIRoot () {}
-    int root_method () const { return -1; } 
+    int root_method () const { return -1; }
 };
 
 
@@ -1198,7 +1301,7 @@ class MIBase1 : public virtual MIRoot
     int m_value;
 public:
     MIBase1 () : m_value (1) {}
-    int base1_method () const { return m_value; } 
+    int base1_method () const { return m_value; }
 };
 
 class MIBase2 : public virtual MIRoot
@@ -1206,16 +1309,16 @@ class MIBase2 : public virtual MIRoot
     int m_value;
 public:
     MIBase2 ()  : m_value (2) {}
-    int base2_method () const { return m_value; } 
+    int base2_method () const { return m_value; }
 };
 
 class MIMixed : public MIBase1, public MIBase2
 {
 public:
     MIMixed () {}
-    int mixed_method () const { return 3; } 
+    int mixed_method () const { return 3; }
 };
-    
+
 
 Tupl my_throwing_func () throw (std::exception);
 
